(function capsule(window) {
  function defineLibrary() {
    const Vec3 = VemaLib.Vec3;
    const AcceleratedBoidsLib = {};
    class Boids {
      constructor(gl, canvas, N, pos) {
        this.random = new Random.Generator(100000004861, 9873347773, 12346561153, 445453344617);
        this.width = 8.0;
        this.height = 4;
        this.offsetx = 0.0 + pos[0];
        this.offsety = 2.5 + pos[1];
        this.offsetz = 0.0 + pos[2];
        this.ground = pos;
        this.center = new Vec3(this.offsetx, this.offsety, this.offsetz);
        this.maxN = N;
        this.N = N;
        this.t = 0;
        this.possi = new Vec3(0.0, 0.0, 0.0);
        this.velli = new Vec3(0.0, 0.0, 1.0);
    // min dist of approach
        this.minDist = 0.1;
    // max magnitude of velocities calculated by "rules"
        this.maxRuleVel = 0.00005;
    // max maginitude of final velocity
        this.maxVel = 0.005;
        this.gridDimension = 25;
        this.gridDimension2 = this.gridDimension * this.gridDimension;
        this.gridDimension3 = this.gridDimension2 * this.gridDimension;
        this.stage = 0;
      }

      init(gl, canvas, textElem, callback) {
        if (this.stage === 0) {
          this.stage++;
          textElem.innerHTML += '<p>Creating Boids Shaders</p>';
          this.createShaders(gl, canvas);
          setTimeout(this.init(gl, canvas, textElem, callback), 10);
          return;
        }
        if (this.stage === 1) {
          this.stage++;
          textElem.innerHTML += '<p>Creating Boids Framebuffers</p>';
          this.createFrameBuffers(gl, canvas);
          setTimeout(this.init(gl, canvas, textElem, callback), 10);
          return;
        }
        if (this.stage === 2) {
          this.stage++;
          textElem.innerHTML += '<p>Creating Boids DataBuffers</p>';
          this.makeDataBuffersData(gl);
          setTimeout(this.init(gl, canvas, textElem, callback), 10);
          return;
        }
        if (this.stage === 3) {
          this.stage++;
          textElem.innerHTML += '<p>Updating vertices</p>';
          this.updateVertices(gl);
          setTimeout(this.init(gl, canvas, textElem, callback), 10);
        }
      }

      tick(gl, canvas, t) {
        this.t = t;
        this.updateVertices(gl);
      }

      createShaders(gl, canvas) {
        const vsBoids = `#version 300 es
precision highp float;
in vec2 position;
out vec2 texXY;
void main() {
  gl_PointSize = 1.0;
  texXY = ((position+1.0)/2.0);
  gl_Position = vec4(position.x,position.y,0.0,1.0);
}`;
        const fsBoids = `#version 300 es
precision highp float;
uniform sampler2D uRandomData;
uniform sampler2D uTextureData;
uniform vec2 uTextureDataSize;
uniform float uMaxRuleVel;
uniform float uMaxVel;
uniform float uMaxRadius;
uniform vec3 uCenter;
uniform vec3 uGround;
uniform vec2 uRandom0;
uniform vec2 uRandom1;
uniform float uMinDist;
in vec2 texXY;
out vec4 fragColor;

vec3 limitVec(vec3 v, float mV){
  float mag = length(v);
  return ((mag>mV)?v*mV/mag:v);
}

float ground(vec2 pos){
  float x = pos.x / 40.0;
  float y = pos.y / 40.0;
  return 40.0*(-1.0 + 0.04 * sin(x * 2.78) + 0.16 * sin(x * 1.58) - 0.14 * sin(y * 0.86) + 0.06 * sin(y * 3.58) + 0.1 * cos((x + y) * 1.34));

}

const float PI = 3.14159265358979323846264;
void main() {
float posy = 0.5/uTextureDataSize.y;
float vely = 1.5/uTextureDataSize.y;
vec4 oldPos = texture(uTextureData,vec2(texXY.x,posy));
vec4 oldVel = texture(uTextureData,vec2(texXY.x,vely));
vec3 pos = oldPos.xyz;
vec3 vel = vec3(0.0,0.0,0.0);

vec3 vel1 = vec3(0.0,0.0,0.0);
vec3 vel2 = vec3(0.0,0.0,0.0);
vec3 vel3 = vec3(0.0,0.0,0.0);

for(float i=0.0; i<uTextureDataSize.x; i++) {
  float x = i/(uTextureDataSize.x-1.0);
 vec3 otherPos = texture(uTextureData, vec2(x,posy)).xyz;

 float distance = length(pos-otherPos);

 if(distance<uMinDist){
   // rule 1 do not get too close to others separation
   vel1+=oldPos.xyz*distance-otherPos*distance;
  }

 if(distance>uMinDist*3.0) continue;

 // rule 2 move same direction as close one alignment
 vel2+=texture(uTextureData,vec2(x,vely)).xyz*distance;

 // rule 3 get close to others. cohesion
 vel3+=(otherPos-oldPos.xyz)*distance;

}

vel+=limitVec(vel1, uMaxRuleVel);
vel+=limitVec(vel2, uMaxRuleVel);
vel+=limitVec(vel3, uMaxRuleVel/2.0);

// rule 4 stay close to center
float n = length(oldPos.xyz-uCenter);
vec3 nv = (oldPos.xyz-uCenter)*(1.0 / n);
vec3 nv2 = nv*2.0;
vec3 opposite = ((nv-nv2)*((n - uMaxRadius) / uMaxRadius * uMaxRuleVel));
vel+=(n > uMaxRadius)?limitVec(opposite, uMaxRuleVel * 2.0):vec3(0.0);

/*
// rule 4 stay inside area
float n = length(oldPos.xyz-uCenter);
vec3 nv = (uCenter-oldPos.xyz)*(n-uMaxRadius);
vel+=(n > uMaxRadius)?limitVec(nv, uMaxRuleVel * 2.0):vec3(0.0);
*/
// rule5 avoid trees
vec3 vel5 = vec3(0.0, 0.0, 0.0);
vec3 direction = vec3(0.0, oldPos.y, 0.0)-oldPos.xyz;
float n0 = length(direction);
float dist = max(0.0,(4.25+uGround.y - oldPos.y)*0.5);
vel5+=(n0 < dist)?(direction*(1.0 / n0 * 0.2 * (-n0))):vec3(0.0,0.0,0.0);
vel+=limitVec(vel5, uMaxRuleVel * 2.0);

// rule 6 make random moves

float angles = PI *2.0* texture(uRandomData,vec2(fract(uRandom0.x+texXY.x),uRandom0.y)).x;
float  anglesup = PI * (texture(uRandomData,vec2(fract(uRandom1.x+texXY.x),uRandom1.y)).x) -PI/2.0;
vec3 vel6 = vec3(cos(angles), sin(anglesup), sin(angles));
vel+=limitVec(vel6,uMaxRuleVel/2.0);

// rule 7 stay above certain level
vec3 vel7 = vec3(0.0, 0.0, 0.0);
vec3 d7 = vec3(oldPos.x, ground(oldPos.xz)+0.1, oldPos.z)-oldPos.xyz;
vel7+=(-d7.y<0.0)?(d7* 5.0 ):vec3(0.0,0.0,0.0);
vel+=limitVec(vel7, uMaxRuleVel * 4.0);

vel = limitVec(vel+oldVel.xyz,uMaxVel);
fragColor = vec4(vel,oldVel.w+ 0.2);
}`;

        this.programBoids = ShaderLib.makeProgram(gl, vsBoids, fsBoids);
        let program = this.programBoids;
        this.paramsBoids = [];
        this.paramsBoids.position = gl.getAttribLocation(program, 'position');
        this.paramsBoids.uRandomData = gl.getUniformLocation(program, 'uRandomData');

        this.paramsBoids.uTextureData = gl.getUniformLocation(program, 'uTextureData');
        this.paramsBoids.uTextureDataSize = gl.getUniformLocation(program, 'uTextureDataSize');
        this.paramsBoids.uMaxRuleVel = gl.getUniformLocation(program, 'uMaxRuleVel');
        this.paramsBoids.uMaxVel = gl.getUniformLocation(program, 'uMaxVel');
        this.paramsBoids.uMaxRadius = gl.getUniformLocation(program, 'uMaxRadius');
        this.paramsBoids.uCenter = gl.getUniformLocation(program, 'uCenter');
        this.paramsBoids.uGround = gl.getUniformLocation(program, 'uGround');
        this.paramsBoids.uRandom0 = gl.getUniformLocation(program, 'uRandom0');
        this.paramsBoids.uRandom1 = gl.getUniformLocation(program, 'uRandom1');

        this.paramsBoids.uMinDist = gl.getUniformLocation(program, 'uMinDist');

        const vsPosition = `#version 300 es
precision highp float;
in vec2 position;
out vec2 texXY;
void main() {
  gl_PointSize = 1.0;
  texXY = ((position+1.0)/2.0);
  gl_Position = vec4(position.x,position.y,0.0,1.0);
}`;
        const fsPosition = `#version 300 es
precision highp float;
uniform sampler2D uTextureData;
uniform vec2 uTextureDataSize;

in vec2 texXY;
out vec4 fragColor;

const float gridDimension = 25.0;
const float gridDimension2 = gridDimension;
const float gridDimension3 = gridDimension2*gridDimension;
const float PI = 3.14159265358979323846264;
void main() {

vec4 oldPos = texture(uTextureData,vec2(texXY.x,0.5/uTextureDataSize.y));
vec4 oldVel = texture(uTextureData,vec2(texXY.x,1.5/uTextureDataSize.y));
vec3 pos = oldPos.xyz;
pos+=oldVel.xyz;
fragColor = vec4(pos,1.0);
}`;

        this.programPosition = ShaderLib.makeProgram(gl, vsPosition, fsPosition);
        program = this.programPosition;
        this.paramsPosition = [];
        this.paramsPosition.position = gl.getAttribLocation(program, 'position');
        this.paramsPosition.uTextureData = gl.getUniformLocation(program, 'uTextureData');
        this.paramsPosition.uTextureDataSize = gl.getUniformLocation(program, 'uTextureDataSize');
      }

      createFrameBuffers(gl, canvas) {
        this.fbB0 = {};
        this.fbB0.texture = gl.createTexture();
        this.fbB0.textureUnit = FrameBufferLib.getNextTextureUnit();
        this.fbB0.framebuffer = FrameBufferLib.createDataFramebufferRGBA(gl, this.fbB0.texture, this.maxN, 0);

        this.fbB1 = {};
        this.fbB1.texture = gl.createTexture();
        this.fbB1.textureUnit = FrameBufferLib.getNextTextureUnit();
        this.fbB1.framebuffer = FrameBufferLib.createDataFramebufferRGBA(gl, this.fbB1.texture, this.maxN, 0);

        this.randomData = new Float32Array(2 * this.N * 256);
        for (let i = 0; i < 2 * this.N * 256; i++) {
          this.randomData[i] = this.random.random();
        }
        this.randomfb = {};
        this.randomfb.texture = gl.createTexture();
        this.randomfb.textureUnit = FrameBufferLib.getNextTextureUnit();
        gl.bindTexture(gl.TEXTURE_2D, this.randomfb.texture);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RG32F, this.N,
            256, 0, gl.RG, gl.FLOAT, this.randomData, 0);

        this.maxTextureSize = gl.getParameter(gl.MAX_TEXTURE_SIZE);
        const theight = Math.ceil((27 * this.N) / this.maxTextureSize);
        const twidth = this.maxTextureSize;

        this.readData = new Float32Array(4 * 2 * 3);
        this.histogram = new Float32Array(this.gridDimension3 * 2);
        this.histogramOffset = new Float32Array(this.gridDimension3);
        this.gridData = new Float32Array(twidth * theight);

        this.nData = new Float32Array(this.N * 384);
        this.nfb = {};
        this.nfb.texture = gl.createTexture();
        this.nfb.textureUnit = FrameBufferLib.getNextTextureUnit();
        gl.bindTexture(gl.TEXTURE_2D, this.nfb.texture);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.R32F, this.N, 384, 0, gl.RED, gl.FLOAT, this.nData);

        this.cfb = this.fbB1;
        this.ofb = this.fbB0;
      }

      makeDataBuffersData(gl) {
        this.vbdata = new Float32Array(8 * this.N * 2);
        this.vbdataBoids = new Float32Array(8 * this.N * 2);
        this.dbdata = new Float32Array(8 * this.N * 4);


        for (let j = 0; j < 8; j++) {
          for (let i = 0; i < this.N; i++) {
            this.vbdata[j * this.N * 2 + i * 2] = -1 + i * 2 / this.N + 1 / this.N;
            this.vbdata[j * this.N * 2 + i * 2 + 1] = -1 + j * 2 / 8 + 1 / 8;
            this.vbdataBoids[j * this.N * 2 + i * 2] = -1 + i * 2 / this.N + 1 / this.N;
            this.vbdataBoids[j * this.N * 2 + i * 2 + 1] = -1 + j * 2 / 8 + 1 / 8;
          }
        }

        for (let i = 0; i < this.N; i++) {
          this.dbdata[i * 4] = (this.random.random() - 0.5) * this.width * 2 + this.offsetx;
          this.dbdata[i * 4 + 1] = (this.random.random() - 0.5) * this.height + this.offsety;
          this.dbdata[i * 4 + 2] = (this.random.random() - 0.5) * this.width * 2 + this.offsetz;
          this.dbdata[i * 4 + 3] = this.random.random() * Math.PI * 2.0 + 0.09;

          const angle = 2.0 * Math.PI * this.random.random();
          const anglesup = -0.2 * Math.PI * this.random.random() + 0.1;
          this.dbdata[this.N * 4 + i * 4] = Math.sin(angle);
          this.dbdata[this.N * 4 + i * 4 + 1] = Math.sin(anglesup);
          this.dbdata[this.N * 4 + i * 4 + 2] = Math.cos(angle);
          this.dbdata[this.N * 4 + i * 4 + 3] = this.random.random() * Math.PI * 2.0;
        }

        gl.bindTexture(gl.TEXTURE_2D, this.fbB0.texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA32F, this.N,
            8, 0, gl.RGBA, gl.FLOAT, this.dbdata, 0);
        gl.bindTexture(gl.TEXTURE_2D, this.fbB1.texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA32F, this.N,
                8, 0, gl.RGBA, gl.FLOAT, this.dbdata, 0);

        this.vbufferData = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbufferData);
        gl.bufferData(gl.ARRAY_BUFFER, this.vbdataBoids, gl.STATIC_DRAW);

        this.dataD = new Float32Array(this.N * this.N);
        let offset = 0;
        for (let j = 0; j < this.N; j++) {
          for (let i = 0; i <= j; i++) {
            this.dataD[offset++] = -1 + i * 2 / (this.N - 1);
            this.dataD[offset++] = -1 + j * 2 / (this.N - 1);
          }
        }
        this.countD = this.dataD.length / 2;
        this.vbufferDistance = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbufferDistance);
        gl.bufferData(gl.ARRAY_BUFFER, this.dataD, gl.STATIC_DRAW);
        this.once = 0;
      }

      updateVertices(gl) {
        const oldfb = gl.getParameter(gl.FRAMEBUFFER_BINDING);
        gl.bindTexture(gl.TEXTURE_2D, this.nfb.texture);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.R32F, this.N, 384, 0, gl.RED, gl.FLOAT, this.nData);
        // Calculate swarm behaviour
        gl.bindFramebuffer(gl.FRAMEBUFFER, this.cfb.framebuffer);
        gl.useProgram(this.programBoids);
        gl.viewport(0, 0, this.N, 8);
        gl.clearColor(0.0, 0.0, 0.0, 1.0);
          // Enable depth testing
        gl.disable(gl.DEPTH_TEST);
          // Clear the color
        gl.clear(gl.COLOR_BUFFER_BIT); // eslint-disable-line no-bitwise

        gl.activeTexture(this.ofb.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.ofb.texture);
        gl.uniform1i(this.paramsBoids.uTextureData, this.ofb.textureUnit.number);
        gl.uniform2f(this.paramsBoids.uTextureDataSize, this.N, 8);
        gl.uniform1f(this.paramsBoids.uMaxRuleVel, this.maxRuleVel);
        gl.uniform1f(this.paramsBoids.uMaxVel, this.maxVel);
        gl.uniform1f(this.paramsBoids.uMaxRadius, this.width / 2.0);
        gl.uniform3f(this.paramsBoids.uCenter, this.center.f[0], this.center.f[1], this.center.f[2]);
        gl.uniform3f(this.paramsBoids.uGround, this.ground[0], this.ground[1], this.ground[2]);
        gl.activeTexture(this.randomfb.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.randomfb.texture);
        gl.uniform1i(this.paramsBoids.uRandomData, this.randomfb.textureUnit.number);
        gl.uniform2f(this.paramsBoids.uRandom0, this.random.random(), this.random.random());
        gl.uniform2f(this.paramsBoids.uRandom1, this.random.random(), this.random.random());
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbufferData);
        gl.enableVertexAttribArray(this.paramsBoids.position);
        gl.vertexAttribPointer(this.paramsBoids.position, 2, gl.FLOAT, false, 0, 0);
        gl.uniform1f(this.paramsBoids.uMinDist, this.minDist);
        gl.drawArrays(gl.POINTS, this.N, this.N);

        gl.useProgram(this.programPosition);
        // render position
        gl.activeTexture(this.ofb.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.ofb.texture);
        gl.uniform1i(this.paramsPosition.uTextureData, this.ofb.textureUnit.number);
        gl.uniform2f(this.paramsPosition.uTextureDataSize, this.N, 8);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbufferData);
        gl.enableVertexAttribArray(this.paramsPosition.position);
        gl.vertexAttribPointer(this.paramsPosition.position, 2, gl.FLOAT, false, 0, 0);

        gl.drawArrays(gl.POINTS, 0, this.N);
        const temp = this.cfb;
        this.cfb = this.ofb;
        this.ofb = temp;

        // gl.readBuffer(gl.COLOR_ATTACHMENT0);
        // gl.readPixels(0, 0, 3, 2, gl.RGBA, gl.FLOAT, this.readData);
        gl.bindFramebuffer(gl.FRAMEBUFFER, oldfb);
      }

      runPhysics(t) {
        this.possi = new Vec3(this.readData[4 * 1 + 0], this.readData[4 * 1 + 1], this.readData[4 * 1 + 2]);
        this.velli = new Vec3(this.readData[4 * 1 + 4 * 3], this.readData[4 * 1 + 4 * 3 + 1], this.readData[4 * 1 + 4 * 3 + 2]);
      }
    }

    AcceleratedBoidsLib.Boids = Boids;
    return AcceleratedBoidsLib;
  }
// define globally if it doesn't already exist
  if (typeof (AcceleratedBoidsLib) === 'undefined') {
    const mwindow = window;
    mwindow.AcceleratedBoidsLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
