(function capsule(window) {
  function defineLibrary() {
    const ButterflyShowLib = {};
    const Vec3 = VemaLib.Vec3; // eslint-disable-line no-undef
    const Matrix4 = VemaLib.Matrix4; // eslint-disable-line no-undef


    let aspectRatio = 1.0;

    ButterflyShowLib.roll = 0.0;
    ButterflyShowLib.zoom = 3.0;
    ButterflyShowLib.offsetx = -0.5;
    ButterflyShowLib.offsety = 0.0;
    ButterflyShowLib.rotate = 0.0;
    ButterflyShowLib.rotateSpeed = 0.0;
    ButterflyShowLib.scrollSpeed = 0.00;
    ButterflyShowLib.spreadSpeed = 0.00;
    ButterflyShowLib.colors = [1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0];
    ButterflyShowLib.colorAdd = [0.0, 0.0, 0.0];

    let program;
    const params = {};
    let ibuffer = null;
    let vbuffer = null;
    let tbuffer = null;
    let nbuffer = null;
    let obuffer = null;
    let pc = 6 * 2;

    let libuffer = null;
    let lvbuffer = null;
    let ltbuffer = null;
    let lnbuffer = null;
    let lobuffer = null;
    let lpc = 6 * 2;

    let programbody;
    const paramsbody = {};
    let ibufferbody = null;
    let vbufferbody = null;
    let tbufferbody = null;
    let pcbody;

    let ibufferantennae = null;
    let vbufferantennae = null;
    let tbufferantennae = null;
    let pcantennae;

    const camerarotation = false;
    const cameraPos = new Vec3(0.0, 1.0, 2.0);
    const cameraDirection = 0.00;
    let modelPos;
    let cameraNormal = new Vec3(0, 0, 1);
    let cameraFront = new Vec3(0, 0, 1); // cpompass direction of camera
    let cameraUp;
    let perspectiveMatrix;
    let cameraMatrix;
    let model;
    let view;
    let normalMat;
    let dirMatrix;

    const fov = 45;
    const viewangle = (35 * Math.PI) / 180;
    const viewdistance = 0.1 / Math.tan(((fov / 2) * Math.PI) / 180);


    ButterflyShowLib.spread = 0.0;
    let wingSpread = 0.0;


    let dataset = null;

    const vertexShader = `#version 300 es
    precision highp float;
    in vec4 vPosition;
    in vec2 multiTexCoord0;
    in float aWingOffset;
    in vec3 aNormal;
    out vec2 texCoord0;
    out vec3 normal;
    uniform mat4 projection;
    uniform mat4 camera;
    uniform mat4 view;
    uniform mat4 model;
    uniform float wingSpread;
    uniform float wingSpeed;
    void main() {
      float wings = sin(wingSpread-aWingOffset*wingSpeed*40.0);
      float ws = ((vPosition.x>0.0)?((wings+0.5)):(-(wings+0.5)));
      mat4 trans = mat4(1.0,0.0,0.0,0.0,
                        0.0,1.0,0.0,0.0,
                        0.0,0.0,1.0,0.0,
                        0.0,-sin(wingSpread)*0.1,0.0,1.0);
      mat4 wsm;
      wsm[0] = vec4(cos(ws), 0.0, -sin(ws), 0.0);
      wsm[1] = vec4(0.0, 1.0, 0.0, 0.0);
      wsm[2] = vec4(sin(ws), 0.0, cos(ws), 0.0);
      wsm[3] = vec4(0.0, 0.0, 0.0, 1.0);
      texCoord0 = multiTexCoord0;
      mat4 nrm;
      nrm[0] = vec4(0.0,0.0, 0.0,0.0);
      nrm[1] = vec4(0.0,sin(ws), cos(ws),0.0);
      nrm[2] = vec4(0.0,-cos(ws),sin(ws),0.0);
      nrm[3] = vec4(0.0,0.0,0.0,1.0);
      vec3 norman = normalize((nrm*wsm*vec4(aNormal,1.0)).xyz);
      normal = (view*model * wsm*vec4(norman,1.0)).xyz;
      gl_Position = projection * camera * view *trans* model * wsm*vPosition;
    }
`;
    const fragmentShader = `#version 300 es
    precision highp float;
    uniform mat4 uRotMatrix;
    in vec2 texCoord0;
    in vec3 normal;
    uniform vec2 centerOffset;
    uniform float zoom;
    uniform float colorroll;
    uniform int colorCount;
    uniform vec3 colors[9];
    uniform vec3 colorAdd;
    out vec4 fragColor;
    const int iterations =255;
    highp float mandel(vec2 tc) {
       highp vec2 c = (zoom*(uRotMatrix*vec4(tc ,0.0,0.0)).xy+centerOffset);
       highp vec2 z = c;
       for (int n = 0; n < iterations; n++) {
           z = vec2(z.x * z.x - z.y * z.y, 2.0 * z.x * z.y) + c;
           if (length(z) > 4.0) return (float(n) + 1.0 - log(log(length(z))) / log(2.0));
       }
       return 0.0;
    }

    void main() {
        highp float n= mandel(texCoord0);
        if (n == 0.0){
          fragColor = vec4(0.0, 0.0, 0.0, 1.0);
        }else{
           n+=colorroll*50.0;
           float value = fract(n*0.02);
           int index0 = int(floor(value*float(colorCount)));
           float f = value*float(colorCount)-float(index0);
           vec3 color = vec3(0.0,0.0,0.0);
           for(int i=0; i<9; i++){
              vec3 c0 = colors[i];
              vec3 c1 = (i<colorCount-1)?colors[i+1]:colors[0];
              color = (i==index0)?mix(c0,c1,f):color;
           }
           vec3 c = color+colorAdd;

           vec3 ambientColor = c.xyz/4.0;
         vec3 diffuseColor = c.xyz;
           vec3 n = normalize(normal);
         vec3 lightDir = normalize(vec3(2.0,20.0,3.0));

           float lambertian = max(abs(dot(n,lightDir)), 0.0);

           vec4 colorLinear = vec4(ambientColor +
                              lambertian * diffuseColor,1.);
           fragColor = colorLinear;
        }
    }
`;

    const vertexShaderBody = `
    precision highp float;
    attribute vec4 vPosition;
    attribute highp vec2 multiTexCoord0;
    varying highp vec2 texCoord0;
    uniform mat4 projection;
    uniform mat4 camera;
    uniform mat4 view;
    uniform mat4 model;
    uniform float wingSpread;
    void main() {
      mat4 trans = mat4(1.0,0.0,0.0,0.0,
                        0.0,1.0,0.0,0.0,
                        0.0,0.0,1.0,0.0,
                        0.0,-sin(wingSpread)*0.1,0.0,1.0);
      texCoord0 = multiTexCoord0;
      gl_Position = projection * camera * view *trans* model *vPosition;
    }
    `;

    const fragmentShaderBody = `
    precision highp float;
    uniform mat4 uRotMatrix;
    varying highp vec2 texCoord0;
    uniform highp vec2 centerOffset;
    uniform highp float zoom;
    uniform highp float colorroll;
    const int iterations =255;
    highp float mandel(vec2 tc) {
       highp vec2 c = (zoom*(uRotMatrix*vec4(tc ,0.0,0.0)).xy+centerOffset);
       highp vec2 z = c;
       for (int n = 0; n < iterations; n++) {
           z = vec2(z.x * z.x - z.y * z.y, 2.0 * z.x * z.y) + c;
           if (length(z) > 4.0) return (float(n) + 1.0 - log(log(length(z))) / log(2.0));
       }
       return 0.0;
    }

    void main() {
        highp float n= mandel(texCoord0);
        if (n == 0.0){
          gl_FragColor = vec4(0.0, 0.0, 0.0, 1.0);
        }else{
           n+=colorroll*50.0;
           vec3 color = vec3(0.95 + 0.012*n , 1.0, 0.2+0.4*(1.0+sin(0.3*n)));
           vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
           vec3 m = abs(fract(color.xxx + K.xyz) * 6.0 - K.www);
           gl_FragColor.rgb = color.z * mix(K.xxx, clamp(m - K.xxx, 0.0, 1.0), color.y)/3.0;
           gl_FragColor.a=1.0;
        }
    }
    `;


    function bc(p0, p1, p2, p3, t) {
      return (1 - t) * (1 - t) * (1 - t) * p0 + 3 * (1 - t) * (1 - t) * t * p1 + 3 * (1 - t) * t * t * p2 + t * t * t * p3;
      // 3 * (1 - t) * (1 - t) * (p1 - p0) + 6 * (1 - t) * t * (p2 - p1) + 3 * t * t * (p3 - p2);
    }

    function makeDataBuffers(gl) {
      if (vbuffer) {
        gl.deleteBuffer(vbuffer);
        vbuffer = null;
      }
      if (tbuffer) {
        gl.deleteBuffer(tbuffer);
        tbuffer = null;
      }
      if (obuffer) {
        gl.deleteBuffer(obuffer);
        obuffer = null;
      }
      if (nbuffer) {
        gl.deleteBuffer(nbuffer);
        nbuffer = null;
      }
      if (ibuffer) {
        gl.deleteBuffer(ibuffer);
        ibuffer = null;
      }


      const vbl = [];
      const vbr = [];
      const ibl = [];
      const ibr = [];
      const tbl = [];
      const tbr = [];
      const obl = [];
      const obr = [];
      const nbl = [];
      const nbr = [];

      const c = dataset.wing.upper.points.length / 2;

      const dc = 20;

      for (let i = 0; i < c; i++) {
        const px = dataset.wing.upper.points[i * 2];
        const py = dataset.wing.upper.points[i * 2 + 1];
        const p = { x: px, y: -py };
        const tp = { x: px - 0.5, y: -(py - 0.5) };
        const woffset = i / c;
        if (i < c - 1) {
          const px2 = dataset.wing.upper.points[(i + 1) * 2];
          const py2 = dataset.wing.upper.points[(i + 1) * 2 + 1];
          const p2 = { x: px2, y: -py2 };
          const tp2 = { x: px2 - 0.5, y: -(py2 - 0.5) };
          const cpp0 = { x: dataset.wing.upper.controlPoints[i * 4], y: dataset.wing.upper.controlPoints[i * 4 + 1] };
          const cpp1 = { x: dataset.wing.upper.controlPoints[i * 4 + 2], y: dataset.wing.upper.controlPoints[i * 4 + 3] };
          const cp0 = { x: cpp0.x, y: -cpp0.y };
          const cp1 = { x: cpp1.x, y: -cpp1.y };
          const tcp0 = { x: cpp0.x - 0.5, y: -(cpp0.y - 0.5) };
          const tcp1 = { x: cpp1.x - 0.5, y: -(cpp1.y - 0.5) };

          for (let j = 0; j < dc; j++) {
            const woffset2 = woffset + 1 / c * j / (dc - 1);
            const st = j / (dc - 1);
            vbl.push(bc(p.x, cp0.x, cp1.x, p2.x, st));
            vbl.push(bc(p.y, cp0.y, cp1.y, p2.y, st));
            vbl.push(0.0);
            vbr.push(-bc(p.x, cp0.x, cp1.x, p2.x, st));
            vbr.push(bc(p.y, cp0.y, cp1.y, p2.y, st));
            vbr.push(0.0);
            tbl.push(bc(tp.x, tcp0.x, tcp1.x, tp2.x, st));
            tbl.push(bc(tp.y, tcp0.y, tcp1.y, tp2.y, st));
            tbr.push(bc(tp.x, tcp0.x, tcp1.x, tp2.x, st));
            tbr.push(bc(tp.y, tcp0.y, tcp1.y, tp2.y, st));
            obl.push(woffset2);
            obr.push(woffset2);
            nbl.push(0.0);
            nbl.push(0.0);
            nbl.push(1.0);
            nbr.push(0.0);
            nbr.push(0.0);
            nbr.push(1.0);
          }
        }
      }

      const offset = (dc * (c - 1));

      for (let i = 0; i < offset; i++) {
        ibl.push(i);
        ibr.push(i + offset);
      }

      const vb = vbl.concat(vbr);
      const tb = tbl.concat(tbr);
      const ib = ibl.concat(ibr);
      const ob = obl.concat(obr);
      const nb = nbl.concat(nbr);

      vbuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, vbuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vb), gl.STATIC_DRAW);
      tbuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, tbuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(tb), gl.STATIC_DRAW);
      obuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, obuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(ob), gl.STATIC_DRAW);
      nbuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, nbuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(nb), gl.STATIC_DRAW);

      ibuffer = gl.createBuffer();
      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, ibuffer);
      gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(ib), gl.STATIC_DRAW);
      pc = ib.length;


      if (lvbuffer) {
        gl.deleteBuffer(lvbuffer);
        lvbuffer = null;
      }
      if (ltbuffer) {
        gl.deleteBuffer(ltbuffer);
        ltbuffer = null;
      }
      if (lobuffer) {
        gl.deleteBuffer(lobuffer);
        lobuffer = null;
      }
      if (lnbuffer) {
        gl.deleteBuffer(lnbuffer);
        lnbuffer = null;
      }
      if (libuffer) {
        gl.deleteBuffer(libuffer);
        libuffer = null;
      }


      const lvbl = [];
      const lvbr = [];
      const libl = [];
      const libr = [];
      const ltbl = [];
      const ltbr = [];
      const lobl = [];
      const lobr = [];
      const lnbl = [];
      const lnbr = [];

      const lc = dataset.wing.lower.points.length / 2;

      // const dc = 20;

      for (let i = 0; i < lc; i++) {
        const px = dataset.wing.lower.points[i * 2];
        const py = dataset.wing.lower.points[i * 2 + 1];
        const p = { x: px, y: -py };
        const tp = { x: px - 0.5, y: -(py - 0.5) };
        const woffset = i / lc;
        if (i < lc - 1) {
          const px2 = dataset.wing.lower.points[(i + 1) * 2];
          const py2 = dataset.wing.lower.points[(i + 1) * 2 + 1];
          const p2 = { x: px2, y: -py2 };
          const tp2 = { x: px2 - 0.5, y: -(py2 - 0.5) };
          const cpp0 = { x: dataset.wing.lower.controlPoints[i * 4], y: dataset.wing.lower.controlPoints[i * 4 + 1] };
          const cpp1 = { x: dataset.wing.lower.controlPoints[i * 4 + 2], y: dataset.wing.lower.controlPoints[i * 4 + 3] };
          const cp0 = { x: cpp0.x, y: -cpp0.y };
          const cp1 = { x: cpp1.x, y: -cpp1.y };
          const tcp0 = { x: cpp0.x - 0.5, y: -(cpp0.y - 0.5) };
          const tcp1 = { x: cpp1.x - 0.5, y: -(cpp1.y - 0.5) };

          for (let j = 0; j < dc; j++) {
            const woffset2 = 0.5 + woffset + 1 / lc * j / (dc - 1);
            const st = j / (dc - 1);
            lvbl.push(bc(p.x, cp0.x, cp1.x, p2.x, st));
            lvbl.push(bc(p.y, cp0.y, cp1.y, p2.y, st));
            lvbl.push(0.0);
            lvbr.push(-bc(p.x, cp0.x, cp1.x, p2.x, st));
            lvbr.push(bc(p.y, cp0.y, cp1.y, p2.y, st));
            lvbr.push(0.0);
            ltbl.push(bc(tp.x, tcp0.x, tcp1.x, tp2.x, st));
            ltbl.push(bc(tp.y, tcp0.y, tcp1.y, tp2.y, st));
            ltbr.push(bc(tp.x, tcp0.x, tcp1.x, tp2.x, st));
            ltbr.push(bc(tp.y, tcp0.y, tcp1.y, tp2.y, st));
            lnbl.push(0.0);
            lnbl.push(0.0);
            lnbl.push(1.0);
            lnbr.push(0.0);
            lnbr.push(0.0);
            lnbr.push(1.0);
            lobl.push(woffset2);
            lobr.push(woffset2);
          }
        }
      }

      const loffset = (dc * (lc - 1));

      for (let i = 0; i < loffset; i++) {
        libl.push(i);
        libr.push(i + loffset);
      }

      const lvb = lvbl.concat(lvbr);
      const ltb = ltbl.concat(ltbr);
      const lib = libl.concat(libr);
      const lob = lobl.concat(lobr);
      const lnb = lnbl.concat(lnbr);

      lvbuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, lvbuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(lvb), gl.STATIC_DRAW);
      ltbuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, ltbuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(ltb), gl.STATIC_DRAW);
      lobuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, lobuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(lob), gl.STATIC_DRAW);
      lnbuffer = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, lnbuffer);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(lnb), gl.STATIC_DRAW);
      libuffer = gl.createBuffer();
      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, libuffer);
      gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(lib), gl.STATIC_DRAW);
      lpc = lib.length;
    }

    function makeDataBuffersBody(gl) {
      if (vbufferbody) {
        gl.deleteBuffer(vbufferbody);
        vbufferbody = null;
      }
      if (tbufferbody) {
        gl.deleteBuffer(tbufferbody);
        tbufferbody = null;
      }
      if (ibufferbody) {
        gl.deleteBuffer(ibufferbody);
        ibufferbody = null;
      }

      const vb = [];
      const tb = [];
      const ib = [];
      const c = dataset.body.points.length / 2;

      const dr = 13;
      const dc = 10;

      for (let i = 0; i < c - 1; i++) {
        const px = dataset.body.points[i * 2];
        const py = dataset.body.points[i * 2 + 1];
        const p = { x: px, y: -py };
        const tp = { x: py - 0.5, y: -(px - 0.5) };
        const px2 = dataset.body.points[(i + 1) * 2];
        const py2 = dataset.body.points[(i + 1) * 2 + 1];
        const p2 = { x: px2, y: -py2 };
        const tp2 = { x: px2 - 0.5, y: -(py2 - 0.5) };
        const cpp0 = { x: dataset.body.controlPoints[i * 4], y: dataset.body.controlPoints[i * 4 + 1] };
        const cpp1 = { x: dataset.body.controlPoints[i * 4 + 2], y: dataset.body.controlPoints[i * 4 + 3] };
        const cp0 = { x: cpp0.x, y: -cpp0.y };
        const cp1 = { x: cpp1.x, y: -cpp1.y };
        const tcp0 = { x: cpp0.x - 0.5, y: -(cpp0.y - 0.5) };
        const tcp1 = { x: cpp1.x - 0.5, y: -(cpp1.y - 0.5) };

        for (let j = 0; j < dc; j++) {
          const st = j / (dc - 1);
          const pq = { x: bc(p.x, cp0.x, cp1.x, p2.x, st),
            y: bc(p.y, cp0.y, cp1.y, p2.y, st) };
          const tpq = { x: bc(tp.x, tcp0.x, tcp1.x, tp2.x, st),
            y: bc(tp.y, tcp0.y, tcp1.y, tp2.y, st) };
          for (let k = 0; k < dr; k++) {
            const a = (k / (dr - 1)) * Math.PI * 2;
            vb.push(Math.sin(a) * pq.x);
            vb.push(pq.y);
            vb.push(Math.cos(a) * pq.x);
            tb.push(tpq.x);
            tb.push(tpq.y);
          }
        }
      }

      for (let i = 0; i < c - 1; i++) {
        for (let k = 0; k < dc - 1; k++) {
          for (let j = 0; j < dr; j++) {
            ib.push(i * dc * dr + k * dr + j);
            ib.push(i * dc * dr + (k + 1) * dr + j);
          }
        }
      }

      vbufferbody = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, vbufferbody);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vb), gl.STATIC_DRAW);
      tbufferbody = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, tbufferbody);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(tb), gl.STATIC_DRAW);
      ibufferbody = gl.createBuffer();
      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, ibufferbody);
      gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(ib), gl.STATIC_DRAW);
      pcbody = ib.length;
    }

    function makeDataBuffersAntennae(gl) {
      if (vbufferantennae) {
        gl.deleteBuffer(vbufferantennae);
        vbufferantennae = null;
      }
      if (tbufferantennae) {
        gl.deleteBuffer(tbufferantennae);
        tbufferantennae = null;
      }
      if (ibufferantennae) {
        gl.deleteBuffer(ibufferantennae);
        ibufferantennae = null;
      }

      if (!dataset.antennae || !dataset.antennae.points) {
        pcantennae = 0;
        return;
      }

      const vbl = [];
      const tbl = [];
      const ibl = [];
      const vbr = [];
      const tbr = [];
      const ibr = [];

      const dr = 11;
      const dc = 11;


      const rootpoint = { x: dataset.body.points[0], y: dataset.body.points[1] };

      // const points = [{ x: 0.00, y: 0.0 }, { x: 0.2, y: -0.3 }];
      const c = dataset.antennae.points.length / 3;
      // const anglex = 0.2;
      // const angley = 0.1;
      const awidth = 0.001;

      // const rotX = new Matrix4();
      // const rotY = new Matrix4();
      const trans = new Matrix4();


      // rotX.rotationXMatrix(anglex);
      // rotY.rotationYMatrix(angley);
      trans.translate(rootpoint.x, -rootpoint.y, 0);
      // const rot = rotY.multiple(rotX.multiple(trans));
      for (let i = 0; i < c - 1; i++) {
        const pp = { x: dataset.antennae.points[i * 3], y: dataset.antennae.points[i * 3 + 1], z: dataset.antennae.points[i * 3 + 2] };
        const pp2 = { x: dataset.antennae.points[(i + 1) * 3], y: dataset.antennae.points[(i + 1) * 3 + 1], z: dataset.antennae.points[(i + 1) * 3 + 2] };
        const p = { x: pp.x, y: pp.y, z: pp.z };
        const p2 = { x: pp2.x, y: pp2.y, z: pp2.z };
        const tp = { x: pp.x - 0.5, y: (pp.y - 0.5) };
        const cpp0 = { x: dataset.antennae.controlPoints[i * 6], y: dataset.antennae.controlPoints[i * 6 + 1], z: dataset.antennae.controlPoints[i * 6 + 2] };
        const cpp1 = { x: dataset.antennae.controlPoints[i * 6 + 3], y: dataset.antennae.controlPoints[i * 6 + 4], z: dataset.antennae.controlPoints[i * 6 + 5] };
        const cp0 = { x: cpp0.x, y: cpp0.y, z: cpp0.z };
        const cp1 = { x: cpp1.x, y: cpp1.y, z: cpp1.z };
        for (let j = 0; j < dc; j++) {
          const ta = j / (dc - 1);
          const ap = new Vec3(bc(p.x, cp0.x, cp1.x, p2.x, ta), bc(p.y, cp0.y, cp1.y, p2.y, ta), bc(p.z, cp0.z, cp1.z, p2.z, ta));
          const tb = (j + 1) / (dc - 1);
          const bp = new Vec3(bc(p.x, cp0.x, cp1.x, p2.x, tb), bc(p.y, cp0.y, cp1.y, p2.y, tb), bc(p.z, cp0.z, cp1.z, p2.z, tb));
          const norm = new Vec3(0.0, 0.0, 1.0);
          const dir = bp.subtract(ap);
          const xcomp = dir.cross(norm).normalized();
          const zcomp = xcomp.cross(dir).normalized();
          for (let k = 0; k < dr; k++) {
            const a = (k / (dr - 1)) * Math.PI * 2;
            // const vp = new Vec3(Math.sin(a) * awidth, p.y, Math.cos(a) * awidth);
            const vp = ap.add(xcomp.multiple(Math.sin(a) * awidth)).add(zcomp.multiple(Math.cos(a) * awidth));
            const v = trans.multipleV(vp);
            vbl.push(v.f[0]);
            vbl.push(v.f[1]);
            vbl.push(v.f[2]);
            tbl.push(0);
            tbl.push(0);
            vbr.push(-v.f[0]);
            vbr.push(v.f[1]);
            vbr.push(v.f[2]);
            tbr.push(0);
            tbr.push(0);
          }
        }
      }

      const offset = (c - 1) * (dc) * dr;
      for (let i = 0; i < (c - 1); i++) {
        for (let j = 0; j < dc - 1; j++) {
          for (let k = 0; k < dr; k++) {
            ibl.push((i * dc * dr) + j * dr + k);
            ibl.push((i * dc * dr) + (j + 1) * dr + k);
            ibr.push(offset + (i * dc * dr) + j * dr + k);
            ibr.push(offset + (i * dc * dr) + (j + 1) * dr + k);
          }
        }
      }

      const vb = vbl.concat(vbr);
      const tb = tbl.concat(tbr);
      const ib = ibl.concat(ibr);

      vbufferantennae = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, vbufferantennae);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vb), gl.STATIC_DRAW);
      tbufferantennae = gl.createBuffer();
      gl.bindBuffer(gl.ARRAY_BUFFER, tbufferantennae);
      gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(tb), gl.STATIC_DRAW);
      ibufferantennae = gl.createBuffer();
      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, ibufferantennae);
      gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(ib), gl.STATIC_DRAW);
      pcantennae = ib.length;
    }

    function makeCamera() {
      perspectiveMatrix = new Matrix4();
      cameraMatrix = new Matrix4();
      model = new Matrix4();
      view = new Matrix4();
      normalMat = new Matrix4();
      modelPos = new Vec3(0.0, 0.0, 0.0);
      const roty = new Matrix4();
      roty.rotationYMatrix(0.234 * ButterflyShowLib.spread - 0.9);
      const rotx = new Matrix4();
      rotx.rotationXMatrix(-Math.PI / 2.5);
      const transy = new Matrix4();
      transy.translate(0, 0.5, 0.5);
      model = transy.multiple(rotx.multiple(roty));
      cameraUp = new Vec3(0, 1, 0);
      cameraFront = new Vec3(0, 0, 1);
      dirMatrix = new Matrix4();
      dirMatrix.rotateMatrix(-cameraDirection, cameraUp);
      cameraFront = dirMatrix.multipleV(cameraFront);
      const rotateAxis = cameraFront.cross(cameraUp).normalized();
      const rot2 = new Matrix4();
      rot2.rotateMatrix(-viewangle, rotateAxis);
      cameraNormal = rot2.multipleV(cameraFront);
      cameraUp = rot2.multipleV(cameraUp);

      cameraMatrix.makeLookAt(cameraNormal, cameraUp, cameraUp.cross(cameraNormal), cameraPos);
      perspectiveMatrix.makePerspective(fov, aspectRatio, viewdistance, 100.0);
    }

    ButterflyShowLib.setData = function setData(data) {
      ButterflyShowLib.zoom = data.zoom;
      ButterflyShowLib.offsetx = data.offsetx;
      ButterflyShowLib.offsety = data.offsety;
      ButterflyShowLib.rotate = data.rotate;
      ButterflyShowLib.colors = data.colors;
      dataset = data;
    };

    ButterflyShowLib.resize = function resize(gl, c) {
      aspectRatio = c.width / c.height;
      // makebuffers(gl);
    };

    ButterflyShowLib.init = function init(gl, c) {
      program = ShaderLib  // eslint-disable-line no-undef
      .makeProgram(gl, vertexShader, fragmentShader);
      gl.useProgram(program);
      params.aPosition = gl.getAttribLocation(program, 'vPosition');
      params.aTexCoordinate = gl.getAttribLocation(program, 'multiTexCoord0');
      params.aWingOffset = gl.getAttribLocation(program, 'aWingOffset');
      params.aNormal = gl.getAttribLocation(program, 'aNormal');
      params.uRotMatrix = gl.getUniformLocation(program, 'uRotMatrix');
      params.colorroll = gl.getUniformLocation(program, 'colorroll');
      params.zoom = gl.getUniformLocation(program, 'zoom');
      params.xy = gl.getUniformLocation(program, 'centerOffset');
      params.projection = gl.getUniformLocation(program, 'projection');
      params.camera = gl.getUniformLocation(program, 'camera');
      params.model = gl.getUniformLocation(program, 'model');
      params.view = gl.getUniformLocation(program, 'view');
      params.wingSpread = gl.getUniformLocation(program, 'wingSpread');
      params.wingSpeed = gl.getUniformLocation(program, 'wingSpeed');
      params.colorAdd = gl.getUniformLocation(program, 'colorAdd');
      params.colorCount = gl.getUniformLocation(program, 'colorCount');
      params.colors = gl.getUniformLocation(program, 'colors');
      programbody = ShaderLib  // eslint-disable-line no-undef
      .makeProgram(gl, vertexShaderBody, fragmentShaderBody);
      gl.useProgram(programbody);
      paramsbody.aPosition = gl.getAttribLocation(programbody, 'vPosition');
      paramsbody.aTexCoordinate = gl.getAttribLocation(programbody, 'multiTexCoord0');
      paramsbody.uRotMatrix = gl.getUniformLocation(programbody, 'uRotMatrix');
      paramsbody.colorroll = gl.getUniformLocation(programbody, 'colorroll');
      paramsbody.zoom = gl.getUniformLocation(programbody, 'zoom');
      paramsbody.xy = gl.getUniformLocation(programbody, 'centerOffset');
      paramsbody.projection = gl.getUniformLocation(programbody, 'projection');
      paramsbody.camera = gl.getUniformLocation(programbody, 'camera');
      paramsbody.model = gl.getUniformLocation(programbody, 'model');
      paramsbody.view = gl.getUniformLocation(programbody, 'view');
      paramsbody.wingSpread = gl.getUniformLocation(programbody, 'wingSpread');

      ButterflyShowLib.resize(gl, c);
    };

    ButterflyShowLib.draw = function draw(gl, canvas) {
      if (dataset) {
        makeDataBuffers(gl);
        makeDataBuffersBody(gl);
        makeDataBuffersAntennae(gl);
        dataset = null;
      }
      makeCamera();
      gl.useProgram(program);
      gl.clearColor(0.3, 0.6, 0.95, 1.0);
      // Enable depth testing
      gl.enable(gl.DEPTH_TEST);
      // Near things obscure far things
      gl.depthFunc(gl.LEQUAL);
      // Clear the color as well as the depth buffer.
      gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT); // eslint-disable-line no-bitwise
      gl.viewport(0, 0, canvas.width, canvas.height);
      gl.bindBuffer(gl.ARRAY_BUFFER, vbuffer);
      gl.enableVertexAttribArray(params.aPosition);
      gl.vertexAttribPointer(params.aPosition, 3, gl.FLOAT, false, 0, 0);
      gl.bindBuffer(gl.ARRAY_BUFFER, nbuffer);
      gl.enableVertexAttribArray(params.aNormal);
      gl.vertexAttribPointer(params.aNormal, 3, gl.FLOAT, false, 0, 0);
      gl.bindBuffer(gl.ARRAY_BUFFER, tbuffer);
      gl.enableVertexAttribArray(params.aTexCoordinate);
      gl.vertexAttribPointer(params.aTexCoordinate, 2, gl.FLOAT, false, 0, 0);

      gl.bindBuffer(gl.ARRAY_BUFFER, obuffer);
      gl.enableVertexAttribArray(params.aWingOffset);
      gl.vertexAttribPointer(params.aWingOffset, 1, gl.FLOAT, false, 0, 0);

      const r = new Matrix4();
      r.rotationZMatrix(ButterflyShowLib.rotate);
      gl.uniformMatrix4fv(params.uRotMatrix, false, new Float32Array(r.m));
      gl.uniform2f(params.xy, ButterflyShowLib.offsetx, ButterflyShowLib.offsety);
      gl.uniform1f(params.zoom, ButterflyShowLib.zoom);
      gl.uniform1f(params.colorroll, ButterflyShowLib.roll);
      gl.uniform1i(params.colorCount, Math.floor(ButterflyShowLib.colors.length / 3));
      gl.uniform3fv(params.colors, new Float32Array(ButterflyShowLib.colors));
      gl.uniform3f(params.colorAdd, ButterflyShowLib.colorAdd[0], ButterflyShowLib.colorAdd[1], ButterflyShowLib.colorAdd[2]);
      gl.uniformMatrix4fv(params.projection, false,
              new Float32Array(perspectiveMatrix.m));
      gl.uniformMatrix4fv(params.camera, false, new Float32Array(cameraMatrix.m));
      gl.uniformMatrix4fv(params.model, false, new Float32Array(model.m));
      gl.uniformMatrix4fv(params.view, false, new Float32Array(view.m));
      gl.uniform1f(params.wingSpread, wingSpread);
      gl.uniform1f(params.wingSpeed, ButterflyShowLib.spreadSpeed);

      ButterflyShowLib.spread += ButterflyShowLib.spreadSpeed;// 0.05;
      wingSpread = ButterflyShowLib.spread;


      ButterflyShowLib.roll += ButterflyShowLib.scrollSpeed;
      if (ButterflyShowLib.roll > 255.0) ButterflyShowLib.roll = 0.0;
      if (ButterflyShowLib.roll < 0.0) ButterflyShowLib.roll = 255.0;

      ButterflyShowLib.rotate += ButterflyShowLib.rotateSpeed;
      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, ibuffer);
      gl.drawElements(gl.TRIANGLE_FAN, pc / 2, gl.UNSIGNED_SHORT, 0);
      gl.drawElements(gl.TRIANGLE_FAN, pc / 2, gl.UNSIGNED_SHORT, pc);

// lower wings

      gl.bindBuffer(gl.ARRAY_BUFFER, lvbuffer);
      gl.enableVertexAttribArray(params.aPosition);
      gl.vertexAttribPointer(params.aPosition, 3, gl.FLOAT, false, 0, 0);
      gl.bindBuffer(gl.ARRAY_BUFFER, ltbuffer);
      gl.enableVertexAttribArray(params.aTexCoordinate);
      gl.vertexAttribPointer(params.aTexCoordinate, 2, gl.FLOAT, false, 0, 0);

      gl.bindBuffer(gl.ARRAY_BUFFER, lobuffer);
      gl.enableVertexAttribArray(params.aWingOffset);
      gl.vertexAttribPointer(params.aWingOffset, 1, gl.FLOAT, false, 0, 0);

      gl.bindBuffer(gl.ARRAY_BUFFER, lnbuffer);
      gl.enableVertexAttribArray(params.aNormal);
      gl.vertexAttribPointer(params.aNormal, 3, gl.FLOAT, false, 0, 0);

      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, libuffer);
      gl.drawElements(gl.TRIANGLE_FAN, lpc / 2, gl.UNSIGNED_SHORT, 0);
      gl.drawElements(gl.TRIANGLE_FAN, lpc / 2, gl.UNSIGNED_SHORT, lpc);

// body rendering
      gl.useProgram(programbody);
      gl.bindBuffer(gl.ARRAY_BUFFER, vbufferbody);
      gl.enableVertexAttribArray(paramsbody.aPosition);
      gl.vertexAttribPointer(paramsbody.aPosition, 3, gl.FLOAT, false, 0, 0);
      gl.bindBuffer(gl.ARRAY_BUFFER, tbufferbody);
      gl.enableVertexAttribArray(paramsbody.aTexCoordinate);
      gl.vertexAttribPointer(paramsbody.aTexCoordinate, 2, gl.FLOAT, false, 0, 0);
      gl.uniformMatrix4fv(paramsbody.uRotMatrix, false, new Float32Array(r.m));
      gl.uniform2f(paramsbody.xy, ButterflyShowLib.offsetx, ButterflyShowLib.offsety);
      gl.uniform1f(paramsbody.zoom, ButterflyShowLib.zoom);
      gl.uniform1f(paramsbody.colorroll, ButterflyShowLib.roll);
      gl.uniform1f(paramsbody.wingSpread, wingSpread);
      gl.uniformMatrix4fv(paramsbody.projection, false,
              new Float32Array(perspectiveMatrix.m));
      gl.uniformMatrix4fv(paramsbody.camera, false, new Float32Array(cameraMatrix.m));
      gl.uniformMatrix4fv(paramsbody.model, false, new Float32Array(model.m));
      gl.uniformMatrix4fv(paramsbody.view, false, new Float32Array(view.m));

      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, ibufferbody);
      gl.drawElements(gl.TRIANGLE_STRIP, pcbody, gl.UNSIGNED_SHORT, 0);

// antennae
      gl.bindBuffer(gl.ARRAY_BUFFER, vbufferantennae);
      gl.enableVertexAttribArray(paramsbody.aPosition);
      gl.vertexAttribPointer(paramsbody.aPosition, 3, gl.FLOAT, false, 0, 0);
      gl.bindBuffer(gl.ARRAY_BUFFER, tbufferantennae);
      gl.enableVertexAttribArray(paramsbody.aTexCoordinate);
      gl.vertexAttribPointer(paramsbody.aTexCoordinate, 2, gl.FLOAT, false, 0, 0);
      gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, ibufferantennae);
      gl.drawElements(gl.TRIANGLE_STRIP, pcantennae / 2, gl.UNSIGNED_SHORT, 0);

      gl.drawElements(gl.TRIANGLE_STRIP, pcantennae / 2, gl.UNSIGNED_SHORT, pcantennae);
    };


    return ButterflyShowLib;
  }
// define globally if it doesn't already exist
  if (typeof (ButterflyShowLib) === 'undefined') {
    const mwindow = window;
    mwindow.ButterflyShowLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
