(function capsule(window) {
  function defineLibrary() {
    // x^3 (-a + b - c + d) + x^2 (2 a - 2 b + c - d) + x (c - a) + b
    function cubicInterpolate(y0, y1, y2, y3, mu) {
      let a0,
        a1,
        a2,
        a3,
        mu2;

      mu2 = mu * mu;
      a0 = y3 - y2 - y0 + y1;
      a1 = y0 - y1 - a0;
      a2 = y2 - y0;
      a3 = y1;
      return (a0 * mu * mu2 + a1 * mu2 + a2 * mu + a3);
    }
    // D((d-c-a+b)*x*x*x + (a-b-(d-c-a+b))*x*x+ (c-a)*x + b)
    // -a + c + 2 (2 a - 2 b + c - d) x + 3 (-a + b - c + d) x^2
    function DerivativeCubicInterpolate(y0, y1, y2, y3, mu) {
      return 3 * mu * mu * (-y0 + y1 - y2 + y3) + 2 * mu * (2 * y0 - 2 * y1 + y2 - y3) - y0 + y2;
    }

    const CamRouteLib = {};
    class CamRoute {

      constructor(numberOfPoints, duration) {
        this.random = new Random.Generator(100000004861, 9873347773, 12346561153, 445453344617);
        this.n = numberOfPoints;
        this.d = duration;
        this.p = this.n - 1;
        this.pd = duration / this.p;
        let a = 0.0;
        this.points = [];
        for (let i = 0; i < this.n; i++) {
          a += Math.PI / 2 + this.random.random() * Math.PI / 4;
          const r = 3.0 + this.random.random() * 6.0;
          const x = r * Math.cos(a);
          const z = r * Math.sin(a);
          const h = 1.0 + 6.0 * this.random.random() + GroundLib.ground(x, z);
          const y = h;
          this.points[i] = [x, y, z];
        }
      }

      getPoint(t) {
        if (t <= 0) {
          return this.points[0];
        }
        if (t >= this.d) {
          return this.points[this.p];
        }
        const i = Math.floor(t / this.pd);
        const f = (t - i * this.pd) / this.pd;
        const a = (i > 0) ? i - 1 : 0;
        const b = i;
        const c = (i < this.n - 1) ? b + 1 : b;
        const d = (i < this.n - 2) ? c + 1 : c;
        const ap = this.points[a];
        const bp = this.points[b];
        const cp = this.points[c];
        const dp = this.points[d];
        return [
          cubicInterpolate(ap[0], bp[0], cp[0], dp[0], f),
          cubicInterpolate(ap[1], bp[1], cp[1], dp[1], f),
          cubicInterpolate(ap[2], bp[2], cp[2], dp[2], f)
        ];
      }

      getDirection(t) {
        if (t <= 0) {
          return this.getDirection(0.001);
        }
        if (t >= this.d) {
          return this.getDirection(this.d - 0.001);
        }

        const i = Math.floor(t / this.pd);
        const f = (t - i * this.pd) / this.pd;
        const a = (i > 0) ? i - 1 : 0;
        const b = i;
        const c = (i < this.n - 1) ? b + 1 : b;
        const d = (i < this.n - 2) ? c + 1 : c;
        const ap = this.points[a];
        const bp = this.points[b];
        const cp = this.points[c];
        const dp = this.points[d];
        return [
          DerivativeCubicInterpolate(ap[0], bp[0], cp[0], dp[0], f),
          DerivativeCubicInterpolate(ap[1], bp[1], cp[1], dp[1], f),
          DerivativeCubicInterpolate(ap[2], bp[2], cp[2], dp[2], f)
        ];
      }

    }

    CamRouteLib.CamRoute = CamRoute;

    return CamRouteLib;
  }
// define globally if it doesn't already exist
  if (typeof (CamRouteLib) === 'undefined') {
    const mwindow = window;
    mwindow.CamRouteLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
