(function capsule(window) {
  function defineLibrary() {
    const DebugLib = {};
    class DebugTexture {
      constructor(gl) {
        const vs = `#version 300 es
    precision highp float;
    in vec2 position;
    out vec2 texXY;
    void main() {
      texXY = ((position+1.0)/2.0);
      gl_Position = vec4(position,1.0,1.0);
    }
    `;
        const fs = `#version 300 es
    precision highp float;
    uniform sampler2D uTextureData;
    uniform vec3 uMultiplier;
    uniform vec3 uOffset;
    in vec2 texXY;
    out vec4 fragColor;
    void main() {
       vec3 c = texture(uTextureData,texXY).xyz*uMultiplier+uOffset;
       fragColor =vec4(c,1.0);
    }
    `;
        this.program = ShaderLib.makeProgram(gl, vs, fs);
        const program = this.program;
        this.params = [];
        this.params.position = gl.getAttribLocation(program, 'position');
        this.params.uTextureData = gl.getUniformLocation(program, 'uTextureData');
        this.params.uMultiplier = gl.getUniformLocation(program, 'uMultiplier');
        this.params.uOffset = gl.getUniformLocation(program, 'uOffset');

        this.vbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1, -1, 1, -1, -1, 1, 1, 1
        ]), gl.STATIC_DRAW);
        this.uMultiplier = 100.0;
        this.uOffset = 0.5;
      }

      draw(gl, texture, w, h, ox, oy) {
        gl.useProgram(this.program);
        gl.viewport(ox, oy, w, h);

        gl.activeTexture(texture.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, texture.texture);
        gl.uniform1i(this.params.uTextureData, texture.textureUnit.number);
        gl.uniform3f(this.params.uMultiplier, this.uMultiplier[0], this.uMultiplier[1], this.uMultiplier[2]);
        gl.uniform3f(this.params.uOffset, this.uOffset[0], this.uOffset[1], this.uOffset[2]);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.enableVertexAttribArray(this.params.position);
        gl.vertexAttribPointer(this.params.position, 2, gl.FLOAT, false, 0, 0);
        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);
      }

      setMultiplier(m) {
        this.uMultiplier = m;
      }
      setOffset(o) {
        this.uOffset = o;
      }
    }
    DebugLib.DebugTexture = DebugTexture;

    return DebugLib;
  }
// define globally if it doesn't already exist
  if (typeof (DebugLib) === 'undefined') {
    const mwindow = window;
    mwindow.DebugLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
