(function capsule(window) {
  function defineLibrary() {
    const FilterLib = {};

    class Filter {

      makebuffers(gl) {
        this.vbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1, -1, 1, -1, -1, 1, 1, 1
        ]), gl.STATIC_DRAW);
        this.tbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([
          0, 0,
          1, 0,
          0, 1,
          1, 1
        ]), gl.STATIC_DRAW);
        this.ibuffer = gl.createBuffer();
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array([0, 1, 2, 1, 2, 3,
        ]), gl.STATIC_DRAW);
      }

      getVertexShader() {
        this.vertexShader = `precision highp float;
  attribute vec4 vPosition;
  attribute highp vec2 texCoordinate;
  varying highp vec2 v_texCoordinate;
  void main() {
    v_texCoordinate = texCoordinate;
    gl_Position = vPosition;
  }`;
      }

      getFragmentShader() {
        this.fragmentShader = `precision highp float;
  uniform sampler2D uTexture0;
  varying highp vec2 v_texCoordinate;

  void main() {
    gl_FragColor = texture2D(uTexture0, v_texCoordinate);
  }`;
      }

      getExtraParams(gl, c) {}

      setExtraParams(gl, c) {}

      constructor(gl, c) {
        this.params = {};
        this.resize(gl, c);
        this.getVertexShader();
        this.getFragmentShader();
        this.program = ShaderLib  // eslint-disable-line no-undef
    .makeProgram(gl, this.vertexShader, this.fragmentShader);
        gl.useProgram(this.program);
        this.params.aPosition = gl.getAttribLocation(this.program, 'vPosition');
        this.params.texCoordinate = gl.getAttribLocation(this.program, 'texCoordinate');
        this.params.uTexture0 = gl.getUniformLocation(this.program, 'uTexture0');
        this.getExtraParams(gl, c);
        this.makebuffers(gl);
      }

      resize(gl, c) {
        this.fb = {};
        this.fb.texture = gl.createTexture();
        this.fb.textureUnit = FrameBufferLib.getNextTextureUnit();
        this.fb.frameBuffer = FrameBufferLib.createFramebuffer(gl, this.fb.texture, c);
      }

      bindFrameBuffer(gl) {
        gl.bindFramebuffer(gl.FRAMEBUFFER, this.fb.frameBuffer);
      }

      draw(gl, canvas, fb) {
        this.bindFrameBuffer(gl);
        gl.useProgram(this.program);
        gl.viewport(0, 0, canvas.width, canvas.height);
        gl.clearColor(0.3, 0.6, 0.95, 1.0);
        // Enable depth testing
        gl.enable(gl.DEPTH_TEST);
        // Near things obscure far things
        gl.depthFunc(gl.LEQUAL);
        // Clear the color as well as the depth buffer.
        gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT); // eslint-disable-line no-bitwise

        gl.activeTexture(fb.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, fb.texture);
        gl.uniform1i(this.params.uTexture0, fb.textureUnit.number);

        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.enableVertexAttribArray(this.params.aPosition);
        gl.vertexAttribPointer(this.params.aPosition, 2, gl.FLOAT, false, 0, 0);

        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.enableVertexAttribArray(this.params.texCoordinate);
        gl.vertexAttribPointer(this.params.texCoordinate, 2, gl.FLOAT, false, 0, 0);
        this.setExtraParams(gl, canvas);

        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);
      }
    }

    FilterLib.Filter = Filter;

    class OutputFilter extends Filter {
      bindFrameBuffer(gl) {
        gl.bindFramebuffer(gl.FRAMEBUFFER, null);
      }
    }

    FilterLib.OutputFilter = OutputFilter;

    return FilterLib;
  }
// define globally if it doesn't already exist
  if (typeof (FilterLib) === 'undefined') {
    const mwindow = window;
    mwindow.FilterLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
