(function capsule(window) {
  function defineLibrary() {
    const Vec3 = VemaLib.Vec3;
    // x^3 (-a + b - c + d) + x^2 (2 a - 2 b + c - d) + x (c - a) + b
    function cubicInterpolate(y0, y1, y2, y3, mu) {
      let a0,
        a1,
        a2,
        a3,
        mu2;

      mu2 = mu * mu;
      a0 = y3 - y2 - y0 + y1;
      a1 = y0 - y1 - a0;
      a2 = y2 - y0;
      a3 = y1;
      return (a0 * mu * mu2 + a1 * mu2 + a2 * mu + a3);
    }
    // D((d-c-a+b)*x*x*x + (a-b-(d-c-a+b))*x*x+ (c-a)*x + b)
    // -a + c + 2 (2 a - 2 b + c - d) x + 3 (-a + b - c + d) x^2
    function DerivativeCubicInterpolate(y0, y1, y2, y3, mu) {
      return 3 * mu * mu * (-y0 + y1 - y2 + y3) + 2 * mu * (2 * y0 - 2 * y1 + y2 - y3) - y0 + y2;
    }

    function ground(xx, yy) {
      const x = xx / 40.0;
      const y = yy / 40.0;
      return 40.0 * (-1.0 + 0.04 * Math.sin(x * 2.78) + 0.16 * Math.sin(x * 1.58) - 0.14 * Math.sin(y * 0.86) + 0.06 * Math.sin(y * 3.58) + 0.1 * Math.cos((x + y) * 1.34));
    }

    function pdXGround(xx, yy) {
      const x = xx / 40.0;
      const y = yy / 40.0;
      return -5.36 * Math.sin(1.34 * (x + y)) + 10.112 * Math.cos(1.58 * x) + 4.448 * Math.cos(2.78 * x);
    }

    function pdYGround(xx, yy) {
      const x = xx / 40.0;
      const y = yy / 40.0;
      return -5.36 * Math.sin(1.34 * (x + y)) - 4.816 * Math.cos(0.86 * y) + 8.592 * Math.cos(3.58 * y);
    }

    /*
     x (-5.36 sin(1.34 (x_0 + y_0)) + 10.112 cos(1.58 x_0) + 4.448 cos(2.78 x_0))
     y (-5.36 sin(1.34 (x_0 + y_0)) - 4.816 cos(0.86 y_0) + 8.592 cos(3.58 y_0))
    */


    function normalGround(x, y) {
      /* const xx = pdXGround(x, y);
      const yy = pdYGround(x, y);
      const xv = new Vec3(1.0, xx, 0.0).normalized();
      const yv = new Vec3(0.0, yy, 1.0).normalized();
      const n = yv.cross(xv).normalized();
*/
      const xx = ground(x + 0.0001, y);
      const yy = ground(x, y + 0.0001);
      const xy = ground(x, y);
      const xv = new Vec3(0.0001, xx - xy, 0.0);
      const yv = new Vec3(0.0, yy - xy, 0.0001);
      const n = xv.cross(yv).normalized();
      return n;
    }

    class Patch {
      constructor(gl, canvas, size, pieces, offset, texture) {
        this.size = size;
        this.p = pieces;
        this.offset = offset;
        this.mTexture = texture;
        // this.createShader(gl);
        this.makeBuffers(gl);
        this.createData(gl);

        this.makeGlBuffers(gl);
      }

      makeBuffers(gl) {
        this.vb = new Float32Array(this.p * this.p * 3);
        this.nb = new Float32Array(this.p * this.p * 3);
        this.tb = new Float32Array(this.p * this.p * 2);
        this.ib = new Uint16Array(this.p * (this.p + 2) * 2);
      }

      createData(gl) {
        for (let j = 0; j < this.p; j++) {
          for (let i = 0; i < this.p; i++) {
            const x = this.offset[0] - this.size / 2.0 + i * this.size / (this.p - 1);
            const y = this.offset[2] - this.size / 2.0 + j * this.size / (this.p - 1);
            this.vb[(j * this.p + i) * 3] = x;
            this.vb[(j * this.p + i) * 3 + 1] = ground(x, y) + this.offset[1];
            this.vb[(j * this.p + i) * 3 + 2] = y;
            const normal = normalGround(x, y);
            this.nb[(j * this.p + i) * 3] = normal.f[0];
            this.nb[(j * this.p + i) * 3 + 1] = normal.f[1];
            this.nb[(j * this.p + i) * 3 + 2] = normal.f[2];
            this.tb[(j * this.p + i) * 2] = i;
            this.tb[(j * this.p + i) * 2 + 1] = j;
          }
        }
        let offset = 0;
        for (let j = 0; j < this.p - 1; j++) {
          this.ib[offset++] = j * this.p;
          for (let i = 0; i < this.p; i++) {
            this.ib[offset++] = j * this.p + i;
            this.ib[offset++] = (j + 1) * this.p + i;
          }
          this.ib[offset] = this.ib[offset - 1];
          offset++;
        }
      }

      makeGlBuffers(gl) {
        this.vbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.vb, gl.STATIC_DRAW);
        this.nbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.nbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.nb, gl.STATIC_DRAW);
        this.tbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.tb, gl.STATIC_DRAW);

        this.ibuffer = gl.createBuffer();
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, this.ib, gl.STATIC_DRAW);
      }


      draw(gl, pm, lm, cm, mm, program, params) {
        gl.useProgram(program);

        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.enableVertexAttribArray(params.aPosition);
        gl.vertexAttribPointer(params.aPosition, 3, gl.FLOAT, false, 0, 0);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.nbuffer);
        gl.enableVertexAttribArray(params.aNormal);
        gl.vertexAttribPointer(params.aNormal, 3, gl.FLOAT, false, 0, 0);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.enableVertexAttribArray(params.aTexCoord);
        gl.vertexAttribPointer(params.aTexCoord, 2, gl.FLOAT, false, 0, 0);

        gl.uniformMatrix4fv(params.projection, false,
                new Float32Array(pm.m));
        gl.uniformMatrix4fv(params.camera, false, new Float32Array(cm.m));
        gl.uniformMatrix4fv(params.model, false, new Float32Array(mm.m));
        gl.uniformMatrix4fv(params.view, false, new Float32Array(lm.m));
        gl.uniform1f(params.wingSpeed, this.spreadSpeed);

        gl.activeTexture(this.mTexture.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.mTexture.texture);
        gl.uniform1i(params.uTexture0, this.mTexture.textureUnit.number);

        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.drawElements(gl.TRIANGLE_STRIP, this.ib.length, gl.UNSIGNED_SHORT, 0);
      }
    }
    class Ground {
      constructor(gl, canvas, size, pieces) {
        this.createShader(gl);
        this.makeTexture(gl);
        this.patches = [];
        for (let i = -2; i < 3; i++) {
          for (let j = -2; j < 3; j++) {
            this.patches.push(new Patch(gl, canvas, size, pieces, [size * j, 0.0, size * i], this.mTexture));
          }
        }
      }

      createShader(gl) {
        const vb = `#version 300 es
      precision highp float;
      in vec3 vPosition;
      in vec3 aNormal;
      in vec2 aTexCoord;
      out vec3 normal;
      out vec2 texCoord;
      uniform mat4 projection;
      uniform mat4 camera;
      uniform mat4 view;
      uniform mat4 model;

      void main() {
      texCoord = aTexCoord;
      normal = (view*model*vec4(aNormal,1.0)).xyz;
      gl_Position = (projection * camera * view * model * vec4(vPosition,1.0));
      }
      `;
        const fb = `#version 300 es
      precision highp float;
      in vec3 normal;
      in vec2 texCoord;
      uniform vec3 colorAdd;
      uniform sampler2D uTexture0;
      out vec4 fragColor;

      void main() {
         vec3 c = texture(uTexture0, texCoord).rgb;
         vec3 ambientColor = c.xyz/4.0;
         vec3 diffuseColor = c.xyz;
         vec3 norm = normalize(normal);
         vec3 lightDir = normalize(vec3(2.0,20.0,10.0));
         float lambertian = max(abs(dot(norm,lightDir)), 0.0);
         vec4 colorLinear = vec4(ambientColor +
                            lambertian * diffuseColor,1.);
         fragColor = colorLinear;

      }
      `;

        this.program = ShaderLib.makeProgram(gl, vb, fb);
        const program = this.program;
        this.params = [];
        this.params.aPosition = gl.getAttribLocation(program, 'vPosition');
        this.params.aNormal = gl.getAttribLocation(program, 'aNormal');
        this.params.aTexCoord = gl.getAttribLocation(program, 'aTexCoord');
        this.params.projection = gl.getUniformLocation(program, 'projection');
        this.params.camera = gl.getUniformLocation(program, 'camera');
        this.params.model = gl.getUniformLocation(program, 'model');
        this.params.view = gl.getUniformLocation(program, 'view');
        this.params.uTexture0 = gl.getUniformLocation(program, 'uTexture0');
      }


      draw(gl, canvas, pm, lm, cm, mm) {
        for (let i = 0; i < this.patches.length; i++) {
          this.patches[i].draw(gl, pm, lm, cm, mm, this.program, this.params);
        }
      }
      makeTexture(gl) {
        this.mTexture = {};
        this.mTexture.texture = gl.createTexture();
        this.mTexture.textureUnit = FrameBufferLib.getNextTextureUnit();
        const can = document.createElement('canvas');
        const ctx = can.getContext('2d');
        can.width = 512;
        can.height = 512;
        ctx.fillStyle = 'darkGreen';
            // ctx.clearRect(0, 0, can.width, can.height);
        ctx.fillRect(0, 0, can.width, can.height);
        for (let j = 0; j < Math.floor(can.width * can.height * 0.3); j++) {
          const green = Math.floor(255.0 * (0.15 + 0.25 * Random.random()));
          const red = Math.floor(255.0 * (0.2 * Random.random()));
          const blue = Math.floor(255.0 * (0.2 * Random.random()));
          ctx.strokeStyle = `rgb(${red},${green},${blue})`;
          ctx.lineWidth = 0.75 + Random.random() * (0.125 * 0.125);
          const x0 = Random.random() * can.width;
          const y0 = Random.random() * can.height;
          const randx = (Random.random()) * 20.0;
          const randy = (Random.random()) * 20.0;
          const x1 = x0 + randx;
          const y1 = y0 + randy;
          ctx.beginPath();
          ctx.moveTo(x0, y0);
          ctx.lineTo(x1, y1);
          ctx.stroke();
          if (x1 > can.width && y1 > can.height) {
            ctx.beginPath();
            ctx.moveTo(x0 - can.width, y0 - can.height);
            ctx.lineTo(x1 - can.width, y1 - can.height);
            ctx.stroke();
          } else if (x1 > can.width) {
            ctx.beginPath();
            ctx.moveTo(x0 - can.width, y0);
            ctx.lineTo(x1 - can.width, y1);
            ctx.stroke();
          } else if (y1 > can.height) {
            ctx.beginPath();
            ctx.moveTo(x0, y0 - can.height);
            ctx.lineTo(x1, y1 - can.height);
            ctx.stroke();
          }
        }
        // document.getElementById('textlayer').appendChild(can);
        gl.activeTexture(this.mTexture.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.mTexture.texture);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.REPEAT);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.REPEAT);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR_MIPMAP_NEAREST);
        gl.texParameterf(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, can);
        gl.generateMipmap(gl.TEXTURE_2D);
      }

    }

    const GroundLib = {};
    GroundLib.ground = ground;
    GroundLib.normal = normalGround;
    GroundLib.Patch = Patch;
    GroundLib.Ground = Ground;
    return GroundLib;
  }
// define globally if it doesn't already exist
  if (typeof (GroundLib) === 'undefined') {
    const mwindow = window;
    mwindow.GroundLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
