(function capsule(window) {
  function defineLibrary() {
    const MatrixLib = {};
    class Matrix extends FilterLib.Filter {

      getVertexShader() {
        this.vertexShader = `#version 300 es
  precision highp float;
  in vec4 vPosition;
  in vec2 texCoordinate;
  out vec2 v_texCoordinate;
  void main() {
    v_texCoordinate = texCoordinate;
    gl_Position = vPosition;
  }`;
      }

      getFragmentShader() {
        this.fragmentShader = `#version 300 es
        precision highp float;
  uniform sampler2D uTexture0;
  uniform sampler2D uCharMapTexture;
  uniform float uSetScalar0;
  uniform int uCharValues[100];
  uniform vec2 uSize;
  uniform float uTime;
  uniform vec2 uCharSize;
  uniform vec4 uColor0;
  uniform vec4 uColor1;
  uniform vec4 uColor2;
  in vec2 v_texCoordinate;
  out vec4 fragmentColor;

  float grayscale(vec4 c){
      return c.r*0.2126+c.g*0.7152+c.b*0.0722;
  }

  vec4 getCharacter(vec2 subCoordinate,float x,float y){
    return texture(uCharMapTexture,vec2((1.0*x)/10.0+subCoordinate.x,1.0-((1.0*y)/10.0+subCoordinate.y)));
  }

  void main () {
      float ratio = uSize.y/uSize.x;
      vec2 rr = vec2(1.0,ratio);
      vec2 ps = uCharSize*(0.1+uSetScalar0*1.9);
      vec2 ccratio = v_texCoordinate*rr-vec2(0.5);
      float coordx =ccratio.x*(uSize.x/ps.x);
      float coordy =ccratio.y*(uSize.x/ps.y);
      float coordxi =float(int(coordx));
      float coordyi =float(int(coordy));
      vec2 camCoordinate = vec2(coordxi,coordyi)/(uSize.x/ps);
      float coordxf = fract(coordx);
      float coordyf = fract(coordy);
      float dx = 1.0/(ps.x*uSize.x/3.0);
      float dy = 1.0/(ps.y*uSize.y/3.0);
      float supers = 0.0;
      for(int i=-1; i<2;i++){
          for(int j=-1; j<2;j++){
              supers += grayscale(texture(uTexture0, (camCoordinate+vec2(0.5)+vec2(float(i)*dx,float(j)*dy))/rr));
          }
      }
      supers/=9.0;
      float mixer =fract(uTime/40.0*(1.0+float((int(coordxi)^0xcafecafe)%int(uSize.y/60.0))/(25.0))+coordyi/2000.0+coordxi/17.0+coordxi*200.0/(71.0*39.0)-float((int(coordxi)^0xdeaddead)%int(uSize.y/30.0))/(100.0));
      vec4 uColorX = (mixer<0.01)?mix(uColor1,uColor0,mixer*100.0):uColor0;
      float character = 0.0;
      for(int i=0; i< 100; i++){
        character = (int(floor(supers*94.0))==i)?(float(uCharValues[i])-((mixer<0.01)?mixer:0.0)):character;
      }
      float x = floor(fract(character/10.0)*10.0);
      float y = 9.0-floor(character/10.0);
      fragmentColor = mix(uColor2,uColorX,getCharacter(vec2(coordxf/10.0,coordyf/10.0),x,y).r);
  }`;
      }

      constructor(gl, canvas) {
        super(gl, canvas);
        this.mColors = [];
        this.mColors[0] = [0.5, 0.9, 0.5, 1.0];
        this.mColors[1] = [0.7, 1.0, 0.7, 1.0];
        this.mColors[2] = [0.0, 0.0, 0.0, 1.0];
        this.time = 0;
        this.fbcharmap = {};
        this.fbcharmap.texture = gl.createTexture();
        this.fbcharmap.textureUnit = FrameBufferLib.getNextTextureUnit();

        const can = document.createElement('canvas');
        const ctx = can.getContext('2d');

        let text = '';
        for (let i = 32; i < 127; i++) {
          text += String.fromCharCode(i);
        }

        let minwidth = 30;
        let minheight = 30;
        for (let i = 0; i < text.length; i++) {
          const tex = String.fromCharCode(i + 32);
          const bounds = ctx.measureText(tex);
          if (minwidth < bounds.width) {
            // minwidth = bounds.width;
          }
          if (minheight < bounds.height) {
            minheight = bounds.height;
          }
        }

        minwidth = Math.ceil(minwidth);
        minheight = Math.ceil(minheight);
        this.minwidth = minwidth;
        this.minheight = minheight;
        const bitmapwidth = 10 * minwidth;
        const bitmapheight = 10 * minheight;
        can.width = bitmapwidth;
        can.height = bitmapheight;
        ctx.fillStyle = 'black';
        ctx.fillRect(0, 0, can.width, can.height);

        ctx.fillStyle = 'white';
        ctx.font = '32px Arial';
        for (let i = 0; i < 10; i++) {
          for (let j = 0; j < 10; j++) {
            const offset = i * 10 + j;
            if (offset < 95) {
              ctx.fillText(text.substring(offset, offset + 1), j * minwidth, can.height - i * minheight);
            }
          }
        }

        // document.getElementById('textlayer').appendChild(can);
        const pixelvalues = [];
        const valueList = [];
        for (let i = 0; i < 10; i++) {
          for (let j = 0; j < 10; j++) {
            const offset = i * 10 + j;
            if (offset < 95) {
              let sum = 0;
              for (let k = 0; k < minwidth; k++) {
                for (let n = 0; n < minheight; n++) {
                  const pixel = ctx.getImageData(j * minwidth + k, i * minheight + n, 1, 1).data;
                  sum += pixel[0];
                }
              }
              pixelvalues[offset] = sum / (255 * minwidth * minheight);
              valueList.push({ letter: text.substring(offset, offset + 1), value: pixelvalues[offset], index: offset, x: j, y: i });
            }
          }
        }

        valueList.sort((a, b) => a.value - b.value);
        this.charValues = [];
        for (let i = 0; i < valueList.length; i++) {
          const lv = valueList[i];
          this.charValues[i] = lv.index;
        }

        gl.activeTexture(this.fbcharmap.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.fbcharmap.texture);

        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
        gl.texParameterf(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, can);
      }

      getExtraParams(gl, canvas) {
        this.params.uCharMapTexture = gl.getUniformLocation(this.program, 'uCharMapTexture');
        this.params.uSetScalar0 = gl.getUniformLocation(this.program, 'uSetScalar0');
        this.params.mCharMapHandle = gl.getUniformLocation(this.program, 'uCharMapTexture');
        this.params.mCharSizeHandle = gl.getUniformLocation(this.program, 'uCharSize');
        this.params.uCharValuesHandle = gl.getUniformLocation(this.program, 'uCharValues');

        this.params.uSize = gl.getUniformLocation(this.program, 'uSize');
        this.params.uTime = gl.getUniformLocation(this.program, 'uTime');

        this.params.uColor0 = gl.getUniformLocation(this.program, 'uColor0');
        this.params.uColor1 = gl.getUniformLocation(this.program, 'uColor1');
        this.params.uColor2 = gl.getUniformLocation(this.program, 'uColor2');
      }

      setExtraParams(gl, canvas) {
        gl.uniform1f(this.params.uSetScalar0, (this.t) / 2.0 + 0.1);

        gl.activeTexture(this.fbcharmap.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.fbcharmap.texture);
        gl.uniform1i(this.params.uCharMapTexture, this.fbcharmap.textureUnit.number);

        gl.uniform2f(this.params.mCharSizeHandle, this.minwidth, this.minheight);
        gl.uniform1iv(this.params.uCharValuesHandle, new Uint16Array(this.charValues));

        gl.uniform2f(this.params.uSize, canvas.width, canvas.height);
        gl.uniform1f(this.params.uTime, this.time);

        gl.uniform4fv(this.params.uColor0, new Float32Array(this.mColors[0]));
        gl.uniform4fv(this.params.uColor1, new Float32Array(this.mColors[1]));
        gl.uniform4fv(this.params.uColor2, new Float32Array(this.mColors[2]));
      }

      resize(gl, c) {
        super.resize(gl, c);
      }

      setT(t) {
        this.t = t;
      }

      setTime(t) {
        this.time = t;
      }

      setColor0(c) {
        this.mColors[0] = c;
      }

    }
    MatrixLib.Matrix = Matrix;
    return MatrixLib;
  }
// define globally if it doesn't already exist
  if (typeof (MatrixLib) === 'undefined') {
    const mwindow = window;
    mwindow.MatrixLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
