(function capsule(window) {
  function defineLibrary() {
    const PencilLib = {};
    class Pencil extends FilterLib.Filter {

      getVertexShader() {
        this.vertexShader = `#version 300 es
  precision highp float;
  in vec4 vPosition;
  in vec2 texCoordinate;
  out vec2 v_TexCoordinate;
  void main() {
    v_TexCoordinate = texCoordinate;
    gl_Position = vPosition;
  }`;
      }

      getFragmentShader() {
        this.fragmentShader = `#version 300 es
        precision highp float;
          uniform sampler2D uTexture0;
          uniform sampler2D uTexture1;
          uniform sampler2D uTexture2;
          uniform sampler2D uTexture3;
          uniform sampler2D uTexture4;
          in vec2 v_TexCoordinate;
          out vec4 fragmentColor;
            vec3 grayscale(vec4 c){
                float g =c.r*0.2126+c.g*0.7152+c.b*0.0722;
                return vec3(g,g,g);
            }

          void main () {
              vec4 cameraColor = texture(uTexture0, v_TexCoordinate);
              vec4 cameraColor1 = texture(uTexture1, v_TexCoordinate*4.0);
              vec4 cameraColor2 = texture(uTexture2, v_TexCoordinate*4.0);
              vec4 cameraColor3 = texture(uTexture3, v_TexCoordinate*4.0);
              vec4 cameraColor4 = texture(uTexture4, v_TexCoordinate*4.0);
              vec4 mixer = vec4(0.0);
              vec3 gray = grayscale(cameraColor);

              mixer = (gray.r<0.875)?cameraColor1:mixer;
              mixer = (gray.r<0.75)?cameraColor2:mixer;
              mixer = (gray.r<0.625)?max(cameraColor1,cameraColor2):mixer;
              mixer = (gray.r<0.5)?cameraColor3:mixer;
              mixer = (gray.r<0.375)?cameraColor4:mixer;
              mixer = (gray.r<0.25)?max(cameraColor3,cameraColor4):mixer;
              mixer = (gray.r<0.125)?max(max(cameraColor1,cameraColor2),max(cameraColor3,cameraColor4)):mixer;

              vec3 finalColor = vec3(1.0)-mix(vec3(0.0),mixer.rgb,mixer.r);
              fragmentColor = vec4(finalColor,1.0);
          }`;
      }

      constructor(gl, canvas) {
        super(gl, canvas);
        this.renderCanvases(gl, canvas);
      }

      getExtraParams(gl, canvas) {
        this.params.uTextures = [];
        this.params.uTextures[0] = gl.getUniformLocation(this.program, 'uTexture1');
        this.params.uTextures[1] = gl.getUniformLocation(this.program, 'uTexture2');
        this.params.uTextures[2] = gl.getUniformLocation(this.program, 'uTexture3');
        this.params.uTextures[3] = gl.getUniformLocation(this.program, 'uTexture4');
      }

      setExtraParams(gl, canvas) {
        for (let i = 0; i < 4; i++) {
          gl.activeTexture(this.mTextures[i].textureUnit.unit);
          gl.bindTexture(gl.TEXTURE_2D, this.mTextures[i].texture);
          gl.uniform1i(this.params.uTextures[i], this.mTextures[i].textureUnit.number);
        }
      }

      renderCanvases(gl, canvas) {
        this.mTextures = [];

        for (let i = 0; i < 4; i++) {
          const fillrate = 0.2 + i * 0.03;

          this.mTextures[i] = {};
          this.mTextures[i].texture = gl.createTexture();
          this.mTextures[i].textureUnit = FrameBufferLib.getNextTextureUnit();
          const can = document.createElement('canvas');
          const ctx = can.getContext('2d');
          can.width = canvas.width / 4;
          can.height = canvas.height / 4;
          ctx.fillStyle = 'black';
              // ctx.clearRect(0, 0, can.width, can.height);
          ctx.fillRect(0, 0, can.width, can.height);
          for (let j = 0; j < Math.floor(can.width * can.height * fillrate); j++) {
            const gray = Math.floor(255.0 * (0.1 + 0.25 * i * (0.5 + 0.5 * Random.random())));
            ctx.strokeStyle = `rgb(${gray},${gray},${gray})`;
            ctx.lineWidth = 0.75 + Random.random() * (0.125 + i * 0.125);
            const x0 = Random.random() * can.width;
            const y0 = Random.random() * can.height;
            const randz = (Random.random() - 0.5) * 15.0 + 10.0;
            const randx = (Random.random() - 0.5) * 2.0;
            const randy = (Random.random() - 0.5) * 2.0;
            let x1 = 0.0;
            let y1 = 0.0;


            switch (i) {
              case 0: {
                x1 += x0 + randx + randz;
                y1 += y0 + randy - randz;

                ctx.beginPath();
                ctx.moveTo(x0, y0);
                ctx.lineTo(x1, y1);
                ctx.stroke();
                if (x1 > can.width && y1 < 0) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0 + can.height);
                  ctx.lineTo(x1 - can.width, y1 + can.height);
                  ctx.stroke();
                } else if (x1 > can.width) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0);
                  ctx.lineTo(x1 - can.width, y1);
                  ctx.stroke();
                } else if (y1 < 0) {
                  ctx.beginPath();
                  ctx.moveTo(x0, y0 + can.height);
                  ctx.lineTo(x1, y1 + can.height);
                  ctx.stroke();
                }
              }
                break;
              case 1: { // combine this shit
                x1 += x0 + randx + randz;
                y1 += y0 + randy + randz;

                ctx.beginPath();
                ctx.moveTo(x0, y0);
                ctx.lineTo(x1, y1);
                ctx.stroke();
                if (x1 > can.width && y1 > can.height) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0 - can.height);
                  ctx.lineTo(x1 - can.width, y1 - can.height);
                  ctx.stroke();
                } else if (x1 > can.width) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0);
                  ctx.lineTo(x1 - can.width, y1);
                  ctx.stroke();
                } else if (y1 > can.height) {
                  ctx.beginPath();
                  ctx.moveTo(x0, y0 - can.height);
                  ctx.lineTo(x1, y1 - can.height);
                  ctx.stroke();
                }
              }
                break;
              case 2: {
                x1 += x0 + randx;
                y1 += y0 + randy + randz;

                ctx.beginPath();
                ctx.moveTo(x0, y0);
                ctx.lineTo(x1, y1);
                ctx.stroke();
                if (x1 > can.width && y1 > can.height) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0 - can.height);
                  ctx.lineTo(x1 - can.width, y1 - can.height);
                  ctx.stroke();
                } else if (x1 > can.width) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0);
                  ctx.lineTo(x1 - can.width, y1);
                  ctx.stroke();
                } else if (y1 > can.height) {
                  ctx.beginPath();
                  ctx.moveTo(x0, y0 - can.height);
                  ctx.lineTo(x1, y1 - can.height);
                  ctx.stroke();
                }
              }
                break;
              case 3: {
                x1 += x0 + randx + randz;
                y1 += y0 + randy;

                ctx.beginPath();
                ctx.moveTo(x0, y0);
                ctx.lineTo(x1, y1);
                ctx.stroke();
                if (x1 > can.width && y1 > can.height) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0 - can.height);
                  ctx.lineTo(x1 - can.width, y1 - can.height);
                  ctx.stroke();
                } else if (x1 > can.width) {
                  ctx.beginPath();
                  ctx.moveTo(x0 - can.width, y0);
                  ctx.lineTo(x1 - can.width, y1);
                  ctx.stroke();
                } else if (y1 > can.height) {
                  ctx.beginPath();
                  ctx.moveTo(x0, y0 - can.height);
                  ctx.lineTo(x1, y1 - can.height);
                  ctx.stroke();
                }
              }
                break;
            }
          }

          // document.getElementById('textlayer').appendChild(can);
          gl.activeTexture(this.mTextures[i].textureUnit.unit);
          gl.bindTexture(gl.TEXTURE_2D, this.mTextures[i].texture);
          gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.REPEAT);
          gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.REPEAT);
          gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
          gl.texParameterf(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
          gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, can);
        }
      }

      resize(gl, c) {
        super.resize(gl, c);
        this.renderCanvases(gl, c);
      }


    }
    PencilLib.Pencil = Pencil;

    class ColorPencil extends Pencil {

      getVertexShader() {
        this.vertexShader = `#version 300 es
  precision highp float;
  in vec4 vPosition;
  in vec2 texCoordinate;
  out vec2 v_TexCoordinate;
  void main() {
    v_TexCoordinate = texCoordinate;
    gl_Position = vPosition;
  }`;
      }

      getFragmentShader2() {
        this.fragmentShader = `#version 300 es
        precision highp float;
  uniform sampler2D uTexture0;
  uniform sampler2D uTexture1;
  uniform sampler2D uTexture2;
  uniform sampler2D uTexture3;
  uniform sampler2D uTexture4;
  uniform float uSetScalar0;
  uniform int iColors0;
  uniform vec4 vColors0[32];
  in vec2 v_TexCoordinate;
  out vec4 fragmentColor;

    float getColorIntensity(vec4 pixel, vec4 color){
        vec3 c =max(pixel.rgb,color.rgb);
        float g =c.r*0.2126+c.g*0.7152+c.b*0.0722;
        return g;
    }

  void main () {
      vec4 cameraColor = texture(uTexture0, v_TexCoordinate);
      vec3 finalColor=vec3(0.0);
      float offset = 1.0/float(iColors0);
      vec4 closestColor=vec4(0.0);
      float d= 100.0;
      for(int i=0; i< iColors0; i+=1){
          float intensity = getColorIntensity(cameraColor, vColors0[i]);

          vec4 cameraColor1 = texture(uTexture1, (v_TexCoordinate+offset)*4.0);
          vec4 cameraColor2 = texture(uTexture2, (v_TexCoordinate+offset)*4.0);
          vec4 cameraColor3 = texture(uTexture3, (v_TexCoordinate+offset)*4.0);
          vec4 cameraColor4 = texture(uTexture4, (v_TexCoordinate+offset)*4.0);

          vec4 mixer = vec4(0.0);
          mixer = (intensity<0.875)?cameraColor1:mixer;
          mixer = (intensity<0.75)?cameraColor2:mixer;
          mixer = (intensity<0.625)?max(cameraColor1,cameraColor2):mixer;
          mixer = (intensity<0.5)?cameraColor3:mixer;
          mixer = (intensity<0.375)?cameraColor4:mixer;
          mixer = (intensity<0.25)?max(cameraColor3,cameraColor4):mixer;
          mixer = (intensity<0.125)?max(max(cameraColor1,cameraColor2),max(cameraColor3,cameraColor4)):mixer;


          float dr = vColors0[i].r-cameraColor.r;
          float dg = vColors0[i].g-cameraColor.g;
          float db = vColors0[i].b-cameraColor.b;
          float di = (dr*dr+dg*dg+db*db)*vColors0[i].a;

          vec3 drawColor = mix(vec3(0.0),vec3(1.0)-vColors0[i].rgb,mixer.r);
          closestColor =(di<d)?vec4(drawColor,1.0):closestColor;
          d= min(di,d);
      }

      fragmentColor = vec4(vec3(1.0)-closestColor.rgb,1.0);
  }`;
      }

      getFragmentShader() {
        this.fragmentShader = `#version 300 es
        precision highp float;
  uniform sampler2D uTexture0;
  uniform sampler2D uTexture1;
  uniform sampler2D uTexture2;
  uniform sampler2D uTexture3;
  uniform sampler2D uTexture4;
  uniform float uSetScalar0;
  uniform int iColors0;
  uniform vec4 vColors0[32];
  in vec2 v_TexCoordinate;
  out vec4 fractalColor;

    float getColorIntensity(vec4 pixel, vec4 color){
        vec3 c =max(pixel.rgb,color.rgb);
        float g =c.r*0.2126+c.g*0.7152+c.b*0.0722;
        return g;
    }

  void main () {
      vec4 cameraColor = texture(uTexture0, v_TexCoordinate);
      vec3 finalColor=vec3(0.0);
      float offset = 1.0/float(iColors0);
      vec4 closestColor=vec4(0.0);
      //float d= 100.0;
      for(int i=0; i< iColors0; i+=1){
          float intensity = getColorIntensity(cameraColor, vColors0[i]);

          vec4 cameraColor1 = texture(uTexture1, (v_TexCoordinate+offset)*4.0);
          vec4 cameraColor2 = texture(uTexture2, (v_TexCoordinate+offset)*4.0);
          vec4 cameraColor3 = texture(uTexture3, (v_TexCoordinate+offset)*4.0);
          vec4 cameraColor4 = texture(uTexture4, (v_TexCoordinate+offset)*4.0);

          vec4 mixer = vec4(0.0);
          mixer = (intensity<0.875)?cameraColor1:mixer;
          mixer = (intensity<0.75)?cameraColor2:mixer;
          mixer = (intensity<0.625)?max(cameraColor1,cameraColor2):mixer;
          mixer = (intensity<0.5)?cameraColor3:mixer;
          mixer = (intensity<0.375)?cameraColor4:mixer;
          mixer = (intensity<0.25)?max(cameraColor3,cameraColor4):mixer;
          mixer = (intensity<0.125)?max(max(cameraColor1,cameraColor2),max(cameraColor3,cameraColor4)):mixer;

          vec3 drawColor = mix(vec3(0.0),vec3(1.0)-vColors0[i].rgb,mixer.r);

          closestColor =(all(lessThanEqual(cameraColor.rgb/2.0,vColors0[i].rgb)))?max(vec4(drawColor,1.0),closestColor):closestColor;

      }
      fractalColor = vec4(vec3(1.0)-closestColor.rgb,1.0);
  }`;
      }

      constructor(gl, canvas) {
        super(gl, canvas);
        this.renderCanvases(gl, canvas);
        const colors = [];
        for (let i = 0; i < 16; i++) {
          colors.push(Random.random());
          colors.push(Random.random());
          colors.push(Random.random());
          colors.push(1.0);
        }
        this.mColors = new Float32Array(colors);
      }

      getExtraParams(gl, canvas) {
        this.params.uTextures = [];
        this.params.uTextures[0] = gl.getUniformLocation(this.program, 'uTexture1');
        this.params.uTextures[1] = gl.getUniformLocation(this.program, 'uTexture2');
        this.params.uTextures[2] = gl.getUniformLocation(this.program, 'uTexture3');
        this.params.uTextures[3] = gl.getUniformLocation(this.program, 'uTexture4');
        this.params.colorCount = gl.getUniformLocation(this.program, 'iColors0');
        this.params.colors = gl.getUniformLocation(this.program, 'vColors0');
      }

      setExtraParams(gl, canvas) {
        for (let i = 0; i < 4; i++) {
          gl.activeTexture(this.mTextures[i].textureUnit.unit);
          gl.bindTexture(gl.TEXTURE_2D, this.mTextures[i].texture);
          gl.uniform1i(this.params.uTextures[i], this.mTextures[i].textureUnit.number);
        }
        gl.uniform1i(this.params.colorCount, 16);
        gl.uniform4fv(this.params.colors, this.mColors);
      }


    }
    PencilLib.ColorPencil = ColorPencil;
    return PencilLib;
  }
// define globally if it doesn't already exist
  if (typeof (PencilLib) === 'undefined') {
    const mwindow = window;
    mwindow.PencilLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
