
(function capsule(window) {
  function defineLibrary() {
    const Matrix4 = VemaLib.Matrix4;
    const SkyLib = {};
    class Sky {
      constructor(gl, w, h, offset) {
        const index = 3;
        this.zoom = 0.0080234;
        this.offsetx = -0.141684591878946;
        this.offsety = -0.8876744014615932;
        this.rotate = -4.60799999999;
        this.colors = [0.3, 0.6, 0.95, 0.7, 0.7, 0.7, 1.0, 1.0, 1.0];

        const vs = `precision highp float;
attribute vec3 vPosition;
attribute highp vec2 multiTexCoord0;
uniform mat4 projection;
uniform mat4 camera;
uniform mat4 view;
uniform mat4 model;
uniform float offset;
varying highp vec2 texCoord0;
void main() {
  texCoord0 = multiTexCoord0;
  gl_Position = projection*view*camera*model*vec4(vPosition+vec3(0.0,offset,0.0),1.0);
}
`;
        const fs = `precision highp float;
uniform mat4 uRotMatrix;
varying highp vec2 texCoord0;
uniform highp vec2 centerOffset;
uniform highp float zoom;
uniform int colorCount;
uniform vec3 colors[9];
const int iterations =255;
const float PI = 3.14159;
highp float mandel(vec2 tc) {
   highp vec2 c = (zoom*(uRotMatrix*vec4(tc ,0.0,0.0)).xy+centerOffset);
   highp vec2 z = c;
   for (int n = 0; n < iterations; n++) {
       z = vec2(z.x * z.x - z.y * z.y, 2.0 * z.x * z.y) + c;
       if (length(z) > 4.0) return (float(n) + 1.0 - log(log(length(z))) / log(2.0));
   }
   return 0.0;
}


void main() {
    highp float n= mandel(texCoord0);
    if (n == 0.0){
      gl_FragColor = vec4(colors[0], 1.0);
    }else{
       vec3 n3 = vec3(n);
       vec3 value = fract(n3*0.02);
       int index0 = int(floor(value.x*float(colorCount)));
       int index1 = int(floor(value.y*float(colorCount)));
       int index2 = int(floor(value.z*float(colorCount)));
       float f0 = value.x*float(colorCount)-float(index0);
       float f1 = value.y*float(colorCount)-float(index1);
       float f2 = value.z*float(colorCount)-float(index2);
       float colorR = 0.0;
       float colorG = 0.0;
       float colorB = 0.0;
       for(int i=0; i<9; i++){
          float c0 = colors[i].r;
          float c1 = (i<colorCount-1)?colors[i+1].r:colors[0].r;
          colorR = (i==index0)?mix(c0,c1,f0):colorR;
          float c2 = colors[i].g;
          float c3 = (i<colorCount-1)?colors[i+1].g:colors[0].g;
          colorG = (i==index1)?mix(c2,c3,f1):colorG;
          float c4 = colors[i].b;
          float c5 = (i<colorCount-1)?colors[i+1].b:colors[0].b;
          colorB = (i==index2)?mix(c4,c5,f2):colorB;
       }
       gl_FragColor.rgb = vec3(colorR,colorG,colorB);
       gl_FragColor.a=1.0;
    }
}`;
        this.program = ShaderLib.makeProgram(gl, vs, fs);
        this.w = w;
        this.h = h;
        this.params = [];
        this.params.aPosition = gl.getAttribLocation(this.program, 'vPosition');
        this.params.aTexCoordinate = gl.getAttribLocation(this.program, 'multiTexCoord0');
        this.params.uRotMatrix = gl.getUniformLocation(this.program, 'uRotMatrix');
        this.params.colorroll = gl.getUniformLocation(this.program, 'colorroll');
        this.params.zoom = gl.getUniformLocation(this.program, 'zoom');
        this.params.xy = gl.getUniformLocation(this.program, 'centerOffset');
        this.params.colorCount = gl.getUniformLocation(this.program, 'colorCount');
        this.params.colors = gl.getUniformLocation(this.program, 'colors');
        this.params.projection = gl.getUniformLocation(this.program, 'projection');
        this.params.camera = gl.getUniformLocation(this.program, 'camera');
        this.params.model = gl.getUniformLocation(this.program, 'model');
        this.params.view = gl.getUniformLocation(this.program, 'view');
        this.params.offset = gl.getUniformLocation(this.program, 'offset');
        this.vbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1000, 10, -1000, 1000, 10, -1000, -1000, 10, 1000, 1000, 10, 1000
        ]), gl.STATIC_DRAW);
        // this.createTexture(gl);
        // this.createCanvas(gl, text);
        this.tbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-0.5, -0.5, 0.5, -0.5, -0.5, 0.5, 0.5, 0.5
        ]), gl.STATIC_DRAW);

        this.ibuffer = gl.createBuffer();
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array([0, 1, 2, 1, 2, 3,
        ]), gl.STATIC_DRAW);
      }

      draw(gl, canvas, pm, cm, lm, mm, offset) {
        gl.disable(gl.DEPTH_TEST);
        gl.useProgram(this.program);
        // gl.viewport(0, 0, canvas.width, canvas.height);
        gl.uniform1i(this.params.uTexture, 0); // todo fix this
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.enableVertexAttribArray(this.params.aPosition);
        gl.vertexAttribPointer(this.params.aPosition, 3, gl.FLOAT, false, 0, 0);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.tbuffer);
        gl.enableVertexAttribArray(this.params.aTexCoordinate);
        gl.vertexAttribPointer(this.params.aTexCoordinate, 2, gl.FLOAT, false, 0, 0);
        const r = new Matrix4();
        r.rotationZMatrix(this.rotate);
        gl.uniformMatrix4fv(this.params.uRotMatrix, false, new Float32Array(r.m));
        gl.uniform2f(this.params.xy, this.offsetx, this.offsety);
        gl.uniform1f(this.params.zoom, this.zoom);
        // gl.uniform1f(this.params.colorroll, this.roll);
        gl.uniform1f(this.params.offset, offset);
        gl.uniform1i(this.params.colorCount, Math.floor(this.colors.length / 3));
        gl.uniform3fv(this.params.colors, new Float32Array(this.colors));

        gl.uniformMatrix4fv(this.params.projection, false,
                new Float32Array(pm.m));
        gl.uniformMatrix4fv(this.params.camera, false, new Float32Array(cm.m));
        gl.uniformMatrix4fv(this.params.model, false, new Float32Array(mm.m));
        gl.uniformMatrix4fv(this.params.view, false, new Float32Array(lm.m));


        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.ibuffer);
        gl.drawElements(gl.TRIANGLES, 6, gl.UNSIGNED_SHORT, 0);
        gl.enable(gl.DEPTH_TEST);
      }
    }

    SkyLib.Sky = Sky;

    return SkyLib;
  }
// define globally if it doesn't already exist
  if (typeof (SkyLib) === 'undefined') {
    const mwindow = window;
    mwindow.SkyLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
