
(function capsule(window) {
  function defineLibrary() {
    const TextBoxLib = {};
    class TextBox {
      constructor(gl, w, h, text) {
        const vs = `#version 300 es
    precision highp float;
    in vec2 position;
    out vec2 texXY;
    uniform float uOffset;
    void main() {
      texXY = vec2(((position.x+1.0)/2.0),1.0-((position.y+1.0)/2.0));
      gl_Position = vec4(position-vec2(2.0*uOffset,0.0),1.0,1.0);
    }
    `;
        const fs = `#version 300 es
    precision highp float;
    uniform sampler2D uTextureData;
    in vec2 texXY;
    out vec4 fragColor;
    void main() {
       fragColor = texture(uTextureData,texXY);
    }
    `;
        this.program = ShaderLib.makeProgram(gl, vs, fs);
        this.w = w;
        this.h = h;
        const program = this.program;
        this.params = [];
        this.params.position = gl.getAttribLocation(program, 'position');
        this.params.uTextureData = gl.getUniformLocation(program, 'uTextureData');
        this.params.uOffset = gl.getUniformLocation(program, 'uOffset');

        this.vbuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1, -1, 1, -1, -1, 1, 1, 1
        ]), gl.STATIC_DRAW);
        this.createTexture(gl);
        this.createCanvas(gl, text);
      }

      createCanvas(gl, text) {
        this.canvas = document.createElement('canvas');
        this.canvas.width = this.w;
        this.canvas.height = this.h;
        this.ctx = this.canvas.getContext('2d');
        this.renderText(gl, text);
      }

      renderText(gl, text) {
        if (text === this.text) return;
        this.text = text;
        this.ctx.fillStyle = 'black';
        this.ctx.fillRect(0, 0, this.w, this.h);
        this.ctx.font = 'bold 20px Times New Roman';
        this.ctx.fillStyle = 'white';
        // this.ctx.fillRect(10, 10, this.w / 2, this.h / 2);
        this.ctx.fillText(this.text, 40, this.h / 2);
        // document.getElementById('textlayer').appendChild(this.canvas);
        this.setTexture(gl);
      }

      createTexture(gl) {
        this.mTexture = {};
        this.mTexture.texture = gl.createTexture();
        this.mTexture.textureUnit = FrameBufferLib.getNextTextureUnit();
      }
      setTexture(gl) {
        gl.activeTexture(this.mTexture.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.mTexture.texture);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.REPEAT);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.REPEAT);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR_MIPMAP_LINEAR);
        gl.texParameterf(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, this.canvas);
        gl.generateMipmap(gl.TEXTURE_2D);
      }

      draw(gl, w, h, ox, oy, offset) {
        gl.disable(gl.DEPTH_TEST);
        gl.useProgram(this.program);
        gl.viewport(ox, oy, w, h);
        gl.activeTexture(this.mTexture.textureUnit.unit);
        gl.bindTexture(gl.TEXTURE_2D, this.mTexture.texture);
        gl.uniform1i(this.params.uTextureData, this.mTexture.textureUnit.number);
        gl.uniform1f(this.params.uOffset, offset);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.vbuffer);
        gl.enableVertexAttribArray(this.params.position);
        gl.vertexAttribPointer(this.params.position, 2, gl.FLOAT, false, 0, 0);
        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);
        gl.enable(gl.DEPTH_TEST);
      }
    }

    TextBoxLib.TextBox = TextBox;

    return TextBoxLib;
  }
// define globally if it doesn't already exist
  if (typeof (TextBoxLib) === 'undefined') {
    const mwindow = window;
    mwindow.TextBoxLib = defineLibrary();
  }
}(window)); // eslint-disable-line no-undef
